"""
Clusto schema

"""

VERSION = 3
from sqlalchemy import MetaData, Table, Column, DDL, TIMESTAMP, func
from sqlalchemy import ForeignKey, String, Integer, Text, Index, DateTime
from sqlalchemy import and_, or_, not_, select, event
from sqlalchemy.exc import OperationalError

from sqlalchemy.orm import scoped_session, sessionmaker, mapper, relation

import sqlalchemy.sql

import sys
import datetime
import clusto
from functools import wraps

try:
    import simplejson as json
except:
    import json


__all__ = ['ATTR_TABLE', 'Attribute', 'and_', 'ENTITY_TABLE', 'Entity', 'func',
           'METADATA', 'not_', 'or_', 'SESSION', 'select', 'VERSION',
           'latest_version', 'CLUSTO_VERSIONING', 'Counter', 'ClustoVersioning',
           'working_version', 'OperationalError', 'ClustoEmptyCommit']


METADATA = MetaData()


CLUSTO_VERSIONING = Table('clustoversioning', METADATA,
                          Column('version', Integer, primary_key=True),
                          Column('timestamp', TIMESTAMP, default=func.current_timestamp(), index=True),
                          Column('user', String(64), default=None),
                          Column('description', Text, default=None),
                          mysql_engine='InnoDB'

                          )

class ClustoEmptyCommit(Exception):
    pass

class ClustoSession(sqlalchemy.orm.interfaces.SessionExtension):

    def after_begin(self, session, transaction, connection):
        if SESSION.clusto_versioning_enabled:
            sql = CLUSTO_VERSIONING.insert().values(user=SESSION.clusto_user,
                                                    description=SESSION.clusto_description)
            session.execute(sql)

        SESSION.clusto_description = None
        SESSION.flushed = set()

    def before_commit(self, session):

        if not any([session.is_modified(x) for x in session]) \
               and hasattr(SESSION, 'flushed') \
               and not SESSION.flushed:
            raise ClustoEmptyCommit()

    def after_commit(self, session):
        SESSION.flushed = set()

    def after_flush(self, session, flush_context):
        SESSION.flushed.update(x for x in session)


SESSION = scoped_session(sessionmaker(autoflush=True, autocommit=True,
                                      extension=ClustoSession()))


def latest_version():
    return select([func.coalesce(func.max(CLUSTO_VERSIONING.c.version), 0)])

def working_version():
    return select([func.coalesce(func.max(CLUSTO_VERSIONING.c.version),1)])

SESSION.clusto_versioning_enabled = True
SESSION.clusto_version = None
SESSION.clusto_user = None
SESSION.clusto_description = None

ENTITY_TABLE = Table('entities', METADATA,
                     Column('entity_id', Integer, primary_key=True),
                     Column('name', String(128, convert_unicode=True),
                            nullable=False, ),
                     Column('type', String(32), nullable=False),
                     Column('driver', String(32), nullable=False),
                     Column('version', Integer, nullable=False),
                     Column('deleted_at_version', Integer, default=None),
                     mysql_engine='InnoDB'
                     )

Index('idx_entity_name_version',
      ENTITY_TABLE.c.name,
      ENTITY_TABLE.c.version,
      ENTITY_TABLE.c.deleted_at_version)

ATTR_TABLE = Table('entity_attrs', METADATA,
                   Column('attr_id', Integer, primary_key=True),
                   Column('entity_id', Integer,
                          ForeignKey('entities.entity_id'), nullable=False),
                   Column('key', String(256, convert_unicode=True),),
                   Column('subkey', String(256, convert_unicode=True),
                          nullable=True,
                          default=None, ),
                   Column('number', Integer, nullable=True, default=None),
                   Column('datatype', String(32), default='string', nullable=False),

                   Column('int_value', Integer, default=None),
                   Column('string_value', Text(convert_unicode=True),
                           default=None,),
                   Column('datetime_value', DateTime, default=None),
                   Column('relation_id', Integer,
                          ForeignKey('entities.entity_id'), default=None),

                   Column('version', Integer, nullable=False),
                   Column('deleted_at_version', Integer, default=None),
                   mysql_engine='InnoDB'

                   )
Index('idx_attrs_entity_version',
      ATTR_TABLE.c.entity_id,
      ATTR_TABLE.c.version,
      ATTR_TABLE.c.deleted_at_version)

Index('idx_attrs_key', ATTR_TABLE.c.key)
Index('idx_attrs_subkey', ATTR_TABLE.c.subkey)

create_index = DDL('CREATE INDEX idx_attrs_str_value on %(table)s (string_value(20))')
event.listen(ATTR_TABLE, 'after_create', create_index.execute_if(dialect='mysql'))

create_index = DDL('CREATE INDEX idx_attrs_str_value on %(table)s ((substring(string_value,0,20)))')
event.listen(ATTR_TABLE, 'after_create', create_index.execute_if(dialect='postgresql'))

create_index = DDL('CREATE INDEX idx_attrs_str_value on %(table)s (string_value)')
event.listen(ATTR_TABLE, 'after_create', create_index.execute_if(dialect='sqlite'))

COUNTER_TABLE = Table('counters', METADATA,
                      Column('counter_id', Integer, primary_key=True),
                      Column('entity_id', Integer, ForeignKey('entities.entity_id'), nullable=False),
                      Column('attr_key', String(256, convert_unicode=True)),
                      Column('value', Integer, default=0),
                      mysql_engine='InnoDB'
                      )

Index('idx_counter_entity_attr',
      COUNTER_TABLE.c.entity_id,
      COUNTER_TABLE.c.attr_key)

class ClustoVersioning(object):
    pass

class Counter(object):

    def __init__(self, entity, keyname, start=0):
        self.entity = entity
        self.attr_key = unicode(keyname)

        self.value = start

        SESSION.add(self)
        SESSION.flush()

    def next(self):

        self.value = Counter.value + 1
        SESSION.flush()
        return self.value

    @classmethod
    def get(cls, entity, keyname, default=0):

        try:
            ctr = SESSION.query(cls).filter(and_(cls.entity==entity,
                                                 cls.attr_key==unicode(keyname))).one()

        except sqlalchemy.orm.exc.NoResultFound:
            ctr = cls(entity, keyname, default)

        return ctr

class ProtectedObj(object):

    ## this is a hack to make these objects immutable-ish
    writable = False

    @staticmethod
    def writer(func):
        @wraps(func)
        def newfunc(self, *args, **kwargs):
            self.writable = True
            res = func(self, *args, **kwargs)
            self.writable = False
            return res
        return newfunc

    def __setattr__(self, name, val):
        if (name.startswith('_sa_')
            or self.writable
            or name == 'writable'):
            super(ProtectedObj, self).__setattr__(name, val)
        else:
            raise Exception("Not Writable")




class Attribute(ProtectedObj):
    """Attribute class holds key/value pair

    An Attribute is a DB backed object that holds a key, number, subkey,
    and a value.

    Each Attribute is associated with an Entity.

    There can be multiple attributes with the same key, number, subkey, and/or
    value.

    Optionally you can explicitely set int_value, string_value,
    datetime_value, relation_id, and datatype.  These settings would override
    the values set by passing in 'value'.
    """

    @ProtectedObj.writer
    def __init__(self, entity, key, value=None,
                 subkey=None, number=None):

        self.entity = entity
        self.key = unicode(key)

        self.value = value

        if subkey is not None:
            subkey = unicode(subkey)
            
        self.subkey = subkey
        self.version = working_version()
        if isinstance(number, bool) and number == True:
            counter = Counter.get(entity, key, default=-1)
            self.number = counter.next()
        elif isinstance(number, Counter):
            self.number = number.next()
        else:
            self.number = number


        SESSION.add(self)
        SESSION.flush()



    def __cmp__(self, other):

        if not isinstance(other, Attribute):
            raise TypeError("Can only compare equality with an Attribute. "
                            "Got a %s instead." % (type(other).__name__))

        return cmp(self.key, other.key)

    def __eq__(self, other):

        if not isinstance(other, Attribute):
            return False

        return ((self.key == other.key) and (self.subkey == other.subkey) and
                (self.value == other.value))

    def __repr__(self):

        params = ('key','value','subkey','number','datatype','version', 'deleted_at_version')
                  #'int_value','string_value','datetime_value','relation_id')


        vals = ((x,getattr(self,x)) for x in params)
        strs = ("%s=%s" % (key, ("'%s'" % val if isinstance(val,basestring) else '%s'%str(val))) for key, val in vals)

        s = "%s(%s)" % (self.__class__.__name__, ','.join(strs))

        return s

    def __str__(self):

        params = ('key','number','subkey','datatype',)

        val = "%s.%s %s" % (self.entity.name, '|'.join([str(getattr(self, param)) for param in params]), str(self.value))
        return val

    @property
    def is_relation(self):
        return self.datatype == 'relation'

    def get_value_type(self, value=None):
        if value == None:
            if self.datatype == None:
                valtype = "string"
            else:
                valtype = self.datatype
        else:
            valtype = self.get_type(value)

        if valtype == 'json':
            valtype = "string"
        return valtype + "_value"

    @property
    def keytuple(self):
        return (self.key, self.number, self.subkey)

    @property
    def to_tuple(self):
        return (self.key, self.number, self.subkey, self.value)

    @classmethod
    def get_type(self, value):

        if isinstance(value, (int,long)):
            if value > sys.maxint:
                raise ValueError("Can only store number between %s and %s"
                                 % (-sys.maxint-1, sys.maxint))
            datatype = 'int'
        elif isinstance(value, basestring):
            datatype = 'string'
        elif isinstance(value, datetime.datetime):
            datatype = 'datetime'
        elif isinstance(value, Entity):
            datatype = 'relation'
        elif hasattr(value, 'entity') and isinstance(value.entity, Entity):
            datatype = 'relation'
        elif isinstance(value, (list, dict)):
            datatype = 'json'
        else:
            datatype = 'string'

        return datatype


    def _get_value(self):

        if self.get_value_type() == 'relation_value':
            return clusto.drivers.base.Driver(getattr(self, self.get_value_type()))
        else:
            val = getattr(self, self.get_value_type())
            if self.datatype == 'int':
                return int(val)
            elif self.datatype == 'json':
                return json.loads(val)
            else:
                return val

    def _set_value(self, value):

        if not isinstance(value, sqlalchemy.sql.ColumnElement):
            self.datatype = self.get_type(value)
            if self.datatype == 'int':
                value = int(value)
            elif self.datatype == 'json':
                value = json.dumps(value)
        
        value_type = self.get_value_type(value)
        
        if value_type == 'string_value' and value is not None:
            value = unicode(value)
        
        setattr(self, value_type, value)

    value = property(_get_value, _set_value)

    @ProtectedObj.writer
    def delete(self):
        ### TODO this seems like a hack

        self.deleted_at_version = working_version()

    @classmethod
    def _version_args(cls):
        args = []
        del_version_args = [cls.deleted_at_version==None]
        if SESSION.clusto_version != None:
          del_version_args.append(cls.deleted_at_version>SESSION.clusto_version)
          args.append(cls.version<=SESSION.clusto_version)


        if len(del_version_args) > 1:
          args.append(or_(*del_version_args))
        else:
          args.append(*del_version_args)

        return args

    @classmethod
    def queryarg(cls, key=None, value=(), subkey=(), number=()):
        args = cls._version_args()

        if key:
            args.append(Attribute.key==unicode(key))

        if number is not ():
            args.append(Attribute.number==number)

        if subkey is not ():
            if subkey is not None:
                subkey = unicode(subkey)
        
            args.append(Attribute.subkey==subkey)

        if value is not ():
            valtype = Attribute.get_type(value) + '_value'
            if valtype == 'relation_value':

                # get entity_id from Drivers too
                if hasattr(value, 'entity'):
                    e = value.entity
                else:
                    e = value

                args.append(getattr(Attribute, 'relation_id') == e.entity_id)
            elif valtype == 'string_value':
                if value is not None:
                    value = unicode(value)
                    
                args.append(getattr(Attribute, 'string_value') == value)
            else:
                args.append(getattr(Attribute, valtype) == value)

        return and_(*args)

    @classmethod
    def query(cls):
        args = cls._version_args()

        return SESSION.query(cls).filter(and_(*args))

class Entity(ProtectedObj):
    """
    The base object that can be stored and managed in clusto.

    An entity can have a name, type, and attributes.

    An Entity's functionality is augmented by Drivers which act as proxies for
    interacting with an Entity and its Attributes.
    """

    @ProtectedObj.writer
    def __init__(self, name, driver='entity', clustotype='entity'):
        """Initialize an Entity.

        @param name: the name of the new Entity
        @type name: C{str}
        @param attrslist: the list of key/value pairs to be set as attributes
        @type attrslist: C{list} of C{tuple}s of length 2
        """

        self.name = unicode(name)

        self.driver = driver
        self.type = clustotype

        self.version = working_version()
        SESSION.add(self)
        SESSION.flush()

    def __eq__(self, otherentity):
        """Am I the same as the Other Entity.

        @param otherentity: the entity you're comparing with
        @type otherentity: L{Entity}
        """

        ## each Thing must have a unique name so I'll just compare those
        if not isinstance(otherentity, Entity):
            retval = False
        else:
            retval = self.name == otherentity.name

        return retval

    def __cmp__(self, other):

        if not hasattr(other, 'name'):
            raise TypeError("Can only compare equality with an Entity-like "
                            "object.  Got a %s instead."
                            % (type(other).__name__))

        return cmp(self.name, other.name)


    def __repr__(self):
        s = "%s(name=%s, driver=%s, clustotype=%s, version=%s, deleted_at_version=%s)"

        return s % (self.__class__.__name__,
                    self.name, self.driver, self.type, str(self.version), str(self.deleted_at_version))

    def __str__(self):
        "Return string representing this entity"

        return str(self.name)

    @property
    def attrs(self):
        return Attribute.query().filter(Attribute.entity==self).all()

    @property
    def references(self):
        return Attribute.query().filter(Attribute.relation_id==self.entity_id).all()


    def add_attr(self, *args, **kwargs):

        return Attribute(self, *args, **kwargs)

    @ProtectedObj.writer
    def delete(self):
        "Delete self and all references to self."

        clusto.begin_transaction()
        try:
            self.deleted_at_version = working_version()

            for i in self.references:
                i.delete()

            for i in self.attrs:
                i.delete()

            clusto.commit()
        except Exception, x:
            clusto.rollback_transaction()
            raise x

    @classmethod
    def _version_args(cls):
        args = []
        del_version_args = [cls.deleted_at_version==None]
        if SESSION.clusto_version != None:
          del_version_args.append(cls.deleted_at_version>SESSION.clusto_version)
          args.append(cls.version<=SESSION.clusto_version)

        args.append(or_(*del_version_args))
        return args

    @classmethod
    def query(cls):
        return SESSION.query(cls).filter(and_(*cls._version_args()))


    @ProtectedObj.writer
    def _set_driver_and_type(self, driver, clusto_type):
        """sets the driver and type for the entity

        this shouldn't be too dangerous, but be careful

        params:
          driver: the driver name
          clusto_type: the type name
        """

        try:
            clusto.begin_transaction()

            self.type = clusto_type
            self.driver = driver

            clusto.commit()
        except Exception, x:
            clusto.rollback_transaction()
            raise x


mapper(ClustoVersioning, CLUSTO_VERSIONING)

mapper(Counter, COUNTER_TABLE,
       properties = {'entity': relation(Entity, lazy=True, uselist=False)},

       )

mapper(Attribute, ATTR_TABLE,
       properties = {'relation_value': relation(Entity, lazy=True,
                                                primaryjoin=ATTR_TABLE.c.relation_id==ENTITY_TABLE.c.entity_id,
                                                uselist=False,
                                                passive_updates=False),
                     'entity': relation(Entity, lazy=True, uselist=False,
                                        primaryjoin=ATTR_TABLE.c.entity_id==ENTITY_TABLE.c.entity_id)})


## might be better to make the relationships here dynamic_loaders in the long
## term.
mapper(Entity, ENTITY_TABLE,

       )





import clusto
from clusto.drivers.base import Driver
from clusto.schema import VERSION

# incrementing the first number means a major schema change
# incrementing the second number means a change in a driver's storage details


class ClustoMeta(Driver):
    """
    Holds meta information about the clusto database
    """

    _properties = {'schemaversion':None}

    _clusto_type = "clustometa"
    _driver_name = "clustometa"


    def __new__(cls):

        try:
            cls.__singleton = clusto.get_by_name(cls._driver_name)
        except LookupError:
            cls.__singleton = Driver.__new__(cls, cls._driver_name)


        return cls.__singleton


    def __init__(self): #, name=None, entity=None, *args, **kwargs):

        if not hasattr(self, 'entity'):
            super(ClustoMeta, self).__init__(self._driver_name)
            self.schemaversion = VERSION




        



#!/usr/bin/env python
# -*- mode: python; sh-basic-offset: 4; indent-tabs-mode: nil; coding: utf-8 -*-
# vim: tabstop=4 softtabstop=4 expandtab shiftwidth=4 fileencoding=utf-8

import argparse
import re
import sys

import clusto
from clusto import script_helper

JSON=False
YAML=False
try:
    import yaml
    YAML=True
except ImportError:
    pass

try:
    import simplejson as json
    JSON=True
except ImportError:
    try:
        import json
        JSON=True
    except:
        pass


class Info(script_helper.Script):
    '''
    This is a script that displays information about a certain object
    (or list of objects) and displays it to stdout.
    '''

    def __init__(self):
        script_helper.Script.__init__(self)

    def format_line(self, key, value, pad=20):
        if value:
            if isinstance(value, list):
                value = ', '.join(value)
            key += ':'
            print key.ljust(pad, ' '), value

    def print_summary(self, items):
        self.debug('Printing with format=summary')
        for item in items:
            self.format_line('Name', item.pop('name'))
            self.format_line('Type', item.pop('type'))
            if 'ip' in item.keys():
                self.format_line('IP', item.pop('ip'))
            self.format_line('Description', item.pop('description'))
            if 'parents' in item.keys():
                self.format_line('Parents', item.pop('parents'))
            if 'contents' in item.keys():
                self.format_line('Contents', item.pop('contents'))
            print '\n'
            keys = sorted(item.keys())
            for k in keys:
                self.format_line(k.capitalize(), item[k])
            print '-' * 80


    def print_oneline(self, items):
        self.debug('Printing with format=oneline')
        for item in items:
            line = '%s(%s);' % (item.pop('name'), item.pop('type'))
            line += '%s;' % (','.join([ '"%s"' % _ for _ in item.pop('description') ]))
            if 'ip' in item.keys():
                line += 'ip=%s;' % (','.join([ _ for _ in item.pop('ip') ]))
            if 'parents' in item.keys():
                line += 'parents=%s;' % (','.join([ _ for _ in item.pop('parents') ]))
            if 'contents' in item.keys():
                line += 'contents=%s;' % (','.join([ _ for _ in item.pop('contents') ]))
            keys = sorted(item.keys())
            for k in keys:
                line += '%s=%s;' % (k, item[k])
            print line

    def print_json(self, items):
        self.debug('Printing with format=json')
        print json.dumps(items, sort_keys=True, indent=2)

    def print_yaml(self, items):
        self.debug('Printing with format=yaml')
        print yaml.safe_dump(items, encoding='utf-8',
            explicit_start=True, default_flow_style=False)

    def run(self, args):
        if not args.items:
            print 'You need to provide at least one item. Use --help'
            return 0
        item_list = []
        self.debug('Fetching the list of items: %s' % ','.join(args.items))
        for item in args.items:
            obj = clusto.get(item)
            if not obj:
                self.warn("The item %s couldn't be found" % item)
                continue
            obj = obj[0]
            self.debug('Object found! %s' % obj)
            item_attrs = {
                'name': obj.name,
                'type': obj.type,
            }
#           Fetch system attrs
            for attr in obj.attrs(key='system'):
                item_attrs[attr.subkey] = attr.value
#           fetch description(s)
            values = obj.attrs(key='description')
            if values:
                item_attrs['description'] = [ _.value for _ in values]
            else:
                item_attrs['description'] = ''
#           fetch parent(s)
            values = obj.parents()
            if values:
                item_attrs['parents'] = [ _.name for _ in values ]
#           fetch content(s)
            values = obj.contents()
            if values:
                item_attrs['contents'] = [ _.name for _ in values ]
#           fetch ip(s)
            if 'get_ips' in dir(obj) and callable(getattr(obj, 'get_ips')):
                values = obj.get_ips()
                if values:
                    item_attrs['ip'] = [ _ for _ in values ]
#           fetch mac(s)
            values = [ _ for _ in obj.attrs(key='port-nic-eth') if _.subkey.find('mac') != -1 ]
            if values:
                for value in values:
                    item_attrs['mac%d' % value.number] = value.value
            item_list.append(item_attrs)
        getattr(self, 'print_%s' % args.format)(item_list)

    def _add_arguments(self, parser):
        choices = ['summary', 'oneline']
        if JSON:
            choices.append('json')
        if YAML:
            choices.append('yaml')
        parser.add_argument('--format', choices=choices, default='summary',
            help='What format to use to display the info, defaults to "summary"')
        parser.add_argument('items', nargs='*', metavar='item',
            help='List of one or more objects to show info')

def main():
    info, args = script_helper.init_arguments(Info)
    return(info.run(args))

if __name__ == '__main__':
    sys.exit(main())





import re
from clusto.drivers.base import Location, Device, Driver

class BasicRack(Location):
    """
    Basic rack driver.
    """

    _clusto_type = "rack"
    _driver_name = "basicrack"

    _properties = {'minu':1,
                   'maxu':45}


    def _ensure_rack_u(self, rackU):
        if not isinstance(rackU, int) and not isinstance(rackU, (list, tuple)):
            raise TypeError("a rackU must be an Integer or list/tuple of Integers.")


        if isinstance(rackU, list):
            for U in rackU:
                if not isinstance(U, int):
                    raise TypeError("a rackU must be an Integer or List of Integers.")

        if isinstance(rackU, int):
            rackU = [rackU]
        else:
            rackU = list(rackU)

        # do U checks
        for U in rackU:
            if U > self.maxu:
                raise TypeError("the rackU must be less than %d." % self.maxu)
            if U < self.minu:
                raise TypeError("RackUs may not be negative.")

        rackU.sort()
        last = rackU[0]
        for i in rackU[1:]:
            if i == last:
                raise TypeError("you can't list the same U twice.")
            if (i-1) != (last):
                raise TypeError("a device can only occupy multiple Us if they're adjacent.")
            last = i

        return rackU

    def _ensure_compatible_device(self, device):
        if not isinstance(device, Device):
            raise TypeError("You can only add Devices to a rack.  %s is a"
                            " %s" % (device.name, str(device.__class__)))

    def insert(self, device, rackU):
        """Insert a given device into the given rackU."""

        self._ensure_compatible_device(device)

        rackU = self._ensure_rack_u(rackU)

        rau = self.get_rack_and_u(device)

        if rau != None:
            raise Exception("%s is already in rack %s"
                            % (device.name, rau['rack'].name))

        if hasattr(device, 'rack_units') and (len(rackU) != device.rack_units):
            raise TypeError("%s is a %dU device, cannot insert it in %dU"
                            % (device.name, device.rack_units, len(rackU)))

        for U in rackU:
            dev = self.get_device_in(U)
            if dev:
                raise TypeError("%s is already in RU %d" % (dev.name, U))

        for U in rackU:
            self.add_attr("_contains", device, number=U, subkey='ru')

    def get_device_in(self, rackU):

        if not isinstance(rackU, int):
            raise TypeError("RackU must be a single integer. Got: %s" % str(rackU))

        rackU = self._ensure_rack_u(rackU)[0]

        owners = self.contents(number=rackU, subkey='ru')

        if len(owners) > 1:
            raise Exception('Somehow there is more than one thing in ru%d.'
                            'Only one of these should be in this space in the '
                            'rack: %s' % (rackU,
                                          ','.join([x.name for x in owners])))
        if owners:
            return owners[0]

        return None

    @classmethod
    def get_rack_and_u(cls, device):
        """
        Get the rack and rackU for a given device.

        returns a tuple of (rack, u-number)
        """

        rack = set(device.parents(clusto_types=[cls]))


        if len(rack) > 1:
            raise Exception("%s is somehow in more than one rack, this will "
                            "likely need to be rectified manually.  It currently "
                            "appears to be in racks %s"
                            % (device.name, str(rack)))

        if rack:
            rack = rack.pop()
            return {'rack':Driver(rack.entity),
                    'RU':[x.number for x in rack.content_attrs(value=device,
                                                              subkey='ru')]}
        else:

            return None



#!/usr/bin/env python
# -*- mode: python; sh-basic-offset: 4; indent-tabs-mode: nil; coding: utf-8 -*-
# vim: tabstop=4 softtabstop=4 expandtab shiftwidth=4 fileencoding=utf-8

import argparse
import sys

import clusto
from clusto import script_helper

class Rename(script_helper.Script):
    '''
    This will rename a given entity
    '''

    def __init__(self):
        script_helper.Script.__init__(self)

    def _add_arguments(self, parser):
        parser.add_argument('oldname', nargs=1,
            help='Name of an entity')
        parser.add_argument('newname', nargs=1,
            help='New name for the given entity')

    def run(self, args):
        try:
            obj = clusto.get_by_name(args.oldname[0])
        except LookupError, e:
            self.debug(e)
            self.error('"%s" does not exist' % args.oldname[0])
            return -1

        try:
            obj = clusto.get_by_name(args.newname[0])
            if obj:
                self.error('There is already an object named "%s"' %
                    args.newname[0])
        except LookupError, e:
            pass

        clusto.rename(obj.name, args.newname[0])
        return


def main():
    rename = Rename()
    parent_parser = script_helper.setup_base_parser()
    this_parser = argparse.ArgumentParser(parents=[parent_parser],
        description=rename._get_description())
    rename._add_arguments(this_parser)
    args = this_parser.parse_args()
    rename.init_script(args=args, logger=script_helper.get_logger(args.loglevel))
    return(rename.run(args))


if __name__ == '__main__':
    sys.exit(main())




from traceback import format_exc
from struct import unpack
from errno import EINTR
from time import time
import socket
import signal

from clusto.services.config import conf, get_logger
log = get_logger('clusto.dhcp', 'INFO')

import logging
runtime = logging.getLogger('scapy.runtime')
runtime.setLevel(logging.ERROR)
loading = logging.getLogger('scapy.loading')
loading.setLevel(logging.ERROR)

from scapy.all import BOOTP, DHCP, DHCPTypes, DHCPOptions, DHCPRevOptions
from IPy import IP

import clusto

extra = conf('dhcp.extra_options')
extra = dict([(int(k), str(v)) for k, v in extra.items()])
DHCPOptions.update(extra)

for k,v in DHCPOptions.iteritems():
    if type(v) is str:
        n = v
        v = None
    else:
        n = v.name
    DHCPRevOptions[n] = (k,v)

class DHCPRequest(object):
    def __init__(self, packet):
        self.packet = packet
        self.parse()

    def parse(self):
        options = self.packet[DHCP].options
        hwaddr = ':'.join(['%02x' % ord(x) for x in self.packet.chaddr[:6]])

        mac = None
        vendor = None
        options = dict([x for x in options if isinstance(x, tuple)])
        if 'client_id' in options:
            mac = unpack('>6s', options['client_id'][1:])[0]
            options['client_id'] = ':'.join(['%02x' % ord(x) for x in mac]).lower()

        self.type = DHCPTypes[options['message-type']]
        self.hwaddr = hwaddr
        self.options = options

class DHCPResponse(object):
    def __init__(self, type, offerip=None, options={}, request=None):
        self.type = type
        self.offerip = offerip
        self.serverip = socket.gethostbyname(socket.gethostname())
        self.options = options
        self.request = request

    def set_type(self, type):
        self.type = type

    def build(self):
        options = [
            ('message-type', self.type)
        ]
        pxelinux = False
        for k, v in self.options.items():
            if k == 'enabled': continue
            if not k in DHCPRevOptions:
                log.warning('Unknown DHCP option: %s' % k)
                continue
            if k.startswith('pxelinux'):
                pxelinux = True
            if isinstance(v, unicode):
                v = v.encode('ascii', 'ignore')
            options.append((k, v))

        if pxelinux:
            options.append(('pxelinux-magic', '\xf1\x00\x75\x7e'))

        bootp_options = {
            'op': 2,
            'xid': self.request.packet.xid,
            'ciaddr': '0.0.0.0',
            'yiaddr': self.offerip,
            'chaddr': self.request.packet.chaddr,
        }
        if 'tftp_server' in self.options:
            bootp_options['siaddr'] = self.options['tftp_server']
        if 'tftp_filename' in self.options:
            bootp_options['file'] = self.options['tftp_filename']
        for k, v in bootp_options.items():
            if isinstance(v, unicode):
                bootp_options[k] = v.encode('ascii', 'ignore')

        pkt = BOOTP(**bootp_options)/DHCP(options=options)
        #pkt.show()
        return pkt.build()

class DHCPServer(object):
    def __init__(self, bind_address=('0.0.0.0', 67)):
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.sock.bind(bind_address)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.server_id = socket.gethostbyname(socket.gethostname())

    def run(self):
        while True:
            try:
                data, address = self.sock.recvfrom(4096)
            except KeyboardInterrupt:
                break
            except socket.error, e:
                if e.args[0] == EINTR:
                    continue
                log.error(format_exc())
                break
            packet = BOOTP(data)
            request = DHCPRequest(packet)

            log.debug('%s %s' % (request.type, request.hwaddr))

            methodname = 'handle_%s' % request.type
            if hasattr(self, methodname):
                method = getattr(self, methodname)
                try:
                    method(request)
                except:
                    log.error(format_exc())
                    continue

    def send(self, address, data):
        while data:
            bytes = self.sock.sendto(str(data), 0, (address, 68))
            data = data[bytes:]

class ClustoDHCPServer(DHCPServer):
    def __init__(self):
        DHCPServer.__init__(self)
        self.offers = {}
        log.info('Clusto DHCP server starting')

    def handle_request(self, request):
        chaddr = request.packet.chaddr
        if not chaddr in self.offers:
            log.warning('Got a request before sending an offer from %s' % request.hwaddr)
            return
        response = self.offers[chaddr]
        response.type = 'ack'

        self.send('255.255.255.255', response.build())

    def handle_discover(self, request):
        if conf('dhcp.update_ipmi'):
            self.update_ipmi(request)

        attrs = [{
            'key': 'port-nic-eth',
            'subkey': 'mac',
            'number': 1,
            'value': request.hwaddr,
        }]
        server = clusto.get_entities(attrs=attrs)

        if not server:
            return

        if len(server) > 1:
            log.warning('More than one server with address %s: %s' % (request.hwaddr, ', '.join([x.name for x in server])))
            return
        
        server = server[0]

        enabled = server.attrs(key='dhcp', subkey='enabled', merge_container_attrs=True)
        if not enabled or not enabled[0].value:
            log.info('DHCP not enabled for %s' % server.name)
            return

        ip = server.attrs(key='ip', subkey='ipstring')
        if not ip:
            log.info('No IP assigned for %s' % server.name)
            return
        else:
            ip = ip[0].value

        ipman = server.attrs(key='ip', subkey='manager')
        if not ipman:
            log.info('Could not find the ip manager for %s' % server.name)
            return
        else:
            ipman = ipman[0].value
        ipman = dict([(x.key, x.value) for x in ipman.attrs(subkey='property')])
        #ipman = dict([(x['key'], x['value']) for x in clusto.get_ip_manager(ip).attrs() if x['subkey'] == 'property'])
        ipy = IP('%s/%s' % (ip, ipman['netmask']), make_net=True)

        options = {
            'server_id': self.server_id,
            'lease_time': 3600,
            'renewal_time': 1600,
            'subnet_mask': ipman['netmask'],
            'broadcast_address': ipy.broadcast().strNormal(),
            'router': ipman['gateway'],
            'hostname': server.hostname,
        }

        log.info('Sending offer to %s, options: %s' % (server.name, options))

        for attr in server.attrs(key='dhcp', merge_container_attrs=True):
            options[attr.subkey] = attr.value

        response = DHCPResponse(type='offer', offerip=ip, options=options, request=request)
        self.offers[request.packet.chaddr] = response
        self.send('255.255.255.255', response.build())

    def update_ipmi(self, request):
        attrs = [{
            'key': 'bootstrap',
            'subkey': 'mac',
            'value': request.hwaddr,
        }, {
            'key': 'port-nic-eth',
            'subkey': 'mac',
            'number': 1,
            'value': request.hwaddr,
        }]
        server = clusto.get_entities(attrs=attrs)

        if not server:
            return

        try:
            server = server[0]
            if request.options.get('vendor_class_id', None) == 'udhcp 0.9.9-pre':
                # This is an IPMI request
                #logging.debug('Associating IPMI %s %s' % (request.hwaddr, server.name))
                server.set_port_attr('nic-eth', 1, 'ipmi-mac', request.hwaddr)
            else:
                #logging.debug('Associating physical %s %s' % (requst.hwaddr, server.name))
                server.set_port_attr('nic-eth', 1, 'mac', request.hwaddr)
        except:
            log.error('Error updating server MAC: %s' % format_exc())





from clusto.drivers.base import Device
from clusto.drivers.devices.common import PortMixin, IPMixin

class BasicNetworkSwitch(IPMixin, PortMixin, Device):
    """
    Basic network switch driver
    """

    _clusto_type = 'networkswitch'
    _driver_name = 'basicnetworkswitch'


    _portmeta = {'pwr-nema-5' : {'numports':1},
                 'nic-eth' : {'numports':48}}






from common import *
from devices import *
from locations import *
from resourcemanagers import *

#from ServerTests import *
#from DatacenterTests import *
from categories import *
#from NetworkTests import *
from resourcetests import *






from clusto.drivers import Device
from clusto.drivers.devices import PortMixin, IPMixin

class BasicConsoleServer(IPMixin, PortMixin, Device):
    """
    Basic console server Driver
    """

    _clusto_type = 'consoleserver'
    _driver_name = 'basicconsoleserver'

    
    _portmeta = { 'pwr-nema-5' : { 'numports':1, },
                  'nic-eth' : { 'numports':1, },
                  'console-serial' : { 'numports':24, },
                  }

    def connect(self, port, num):
        raise NotImplemented



import clusto
from clusto.drivers.base import ResourceManager

from clusto.exceptions import ResourceException
from clusto.schema import ATTR_TABLE

class SimpleNumManagerException(ResourceException):
    pass

class SimpleNumManager(ResourceManager):
    """Manage the generation of numbers that can be associated with Entities
    
    """

    _driver_name = "simplenummanager"
    _properties = {'maxnum':None,
                   'next':0,
                   }

    _record_allocations = True
    _attr_name = "simplenum"
    
    def allocator(self, thing=None):

        clusto.flush()

        counter = clusto.Counter.get(self.entity, 'next', default=self.next)

        num = counter.value
        
        if self.maxnum and num > self.maxnum:
            raise SimpleNumManagerException("Out of numbers. "
                                            "Max of %d reached." 
                                            % (self.maxnum))
        
        counter.next()
        return (num, True)




from clusto.test import testbase

from clusto.drivers import BasicServer, BasicRack, BasicPowerStrip, BasicNetworkSwitch, BasicDatacenter, IPManager, Pool
from clusto.exceptions import ConnectionException
import clusto

class ClusterUsageTest(testbase.ClustoTestBase):
    """Test managing a cluster

    create pools, find services, query machine properties, etc.
    """

    def data(self):

        def createRack(datacenter, rackprefix):

            try:
                clusto.begin_transaction()
                r = BasicRack(rackprefix)
                pwr = BasicPowerStrip(rackprefix+'-pwr1', withslave=True)

                sw = BasicNetworkSwitch(rackprefix+'-sw1')
                sw.connect_ports('nic-eth', 48, pwr, 1)
                pwr.connect_ports('pwr-nema-5', 1, sw, 1)

                r.insert(pwr, [1,2,3,4])
                r.insert(sw, [5])

                for i in range(20):
                    s=BasicServer(rackprefix+'-s'+'%02d'%i)
                    r.insert(s, [6+i])
                    s.connect_ports('nic-eth', 1, sw, i+1)
                    s.connect_ports('pwr-nema-5', 1,
                                    pwr, i+2)
                clusto.commit()
            except Exception, x:
                clusto.rollback_transaction()
                raise x
                
            return r

        ds = map(BasicDatacenter, ['dc1', 'dc2', 'dc3'])

        for num, d in enumerate(ds):
            for i in range(1):
                rackname = 'rack-'+ str(num) + '%03d' % i
                r = createRack(d, rackname)

                d.insert(r)

        ipmans = [IPManager('block-' + x, netmask='255.255.0.0', baseip=x)
                  for x in ('10.1.0.0', '10.2.0.0', '10.3.0.0')]

        state_pools = map(Pool, ('production', 'development'))
        type_pools = map(Pool, ('webserver', 'database'))
        db_group_pools = map(Pool, ('users', 'objects', 'logs'))
        web_group_pools = map(Pool, ('frontend', 'api', 'admin'))
        
        for num, d in enumerate(ds):
            ipman = ipmans[num]
            for s in d.contents(clusto_types=[BasicServer],
                                search_children=True):
                ipman.allocate(s)
                ipman.allocate(s)
                
        
    def testGetServers(self):
        pass



#!/usr/bin/env python
# -*- mode: python; sh-basic-offset: 4; indent-tabs-mode: nil; coding: utf-8 -*-
# vim: tabstop=4 softtabstop=4 expandtab shiftwidth=4 fileencoding=utf-8
#
# Shell command
# Copyright 2010, Jeremy Grosser <synack@digg.com>

import argparse
import os
import sys

import clusto
from clusto import script_helper


class Console(script_helper.Script):
    '''
    Use clusto's hardware port mappings to console to a remote server
    using the serial console.
    '''

    def __init__(self):
        script_helper.Script.__init__(self)

    def _add_arguments(self, parser):
        parser.add_argument('--user', '-u',
            help='SSH User (you can also set this in clusto.conf '
                 'in console.user: --user > clusto.conf:console.user')
        parser.add_argument('server', nargs=1,
            help='Object to console to (IP or name)')

    def run(self, args):
        try:
            server = clusto.get(args.server[0])
            if not server:
                raise LookupError('Object "%s" does not exist' % args.server)
        except Exception as e:
            self.debug(e)
            self.error('No object like "%s" was found' % args.server)
            return 1
        server = server[0]

        if not hasattr(server, 'console'):
            self.error('The object %s lacks a console method' % server.name)
            return 2

        user = os.environ.get('USER')
        if args.user:
            self.debug('Grabbing user from parameter')
            user = args.user
        else:
            self.debug('Grabbing user from config file or default')
            user = self.get_conf('console.user', user)
        self.debug('User is "%s"' % user)
        return(server.console(ssh_user=user))


def main():
    console, args = script_helper.init_arguments(Console)
    return(console.run(args))

if __name__ == '__main__':
    sys.exit(main())





from clusto.drivers import Device
from clusto.drivers.devices import PortMixin, IPMixin

class BasicAppliance(IPMixin, PortMixin, Device):
    """
    Basic appliance Driver
    """

    _clusto_type = 'appliance'
    _driver_name = 'basicappliance'

    
    _portmeta = { 'pwr-nema-5' : { 'numports':2, },
                  'nic-eth' : { 'numports':1, },
                  'console-serial' : { 'numports':1, },
                  }




from clusto.drivers.base import Driver

class Location(Driver):

    _clusto_type = "location"
    _driver_name = "location"

    def insert(self, thing):
        """Insert the given Enity or Driver into this Entity.

        Such that:

        >>> A.insert(B)
        >>> (B in A)
        True

        A given entity can only be in one Location one time.
        """

        d = self.ensure_driver(thing,
                               "Can only insert an Entity or a Driver. "
                               "Tried to insert %s." % str(type(thing)))

        locations = thing.parents(clusto_drivers=[self._driver_name])
        if locations:
            raise TypeError("%s is already in location(s) %s." %
                                (thing, locations))

        self.add_attr("_contains", d, number=True)



import clusto
from clusto.drivers.base import ResourceManager

from clusto.exceptions import ResourceException
from clusto.schema import ATTR_TABLE

class SimpleNameManagerException(ResourceException):
    pass

class SimpleNameManager(ResourceManager):
    """
    SimpleNameManager - manage the generation of a names with a common
    prefix and an incrementing integer component.

    e.g foo001, foo002, foo003, etc.
    
    """

    _driver_name = "simplenamemanager"
    _properties = {'basename':'',
                   'digits':2,
                   'next':1,
                   'leadingZeros':int(True)}

    _record_allocations = True
    _attr_name = 'simplename'
    
    def allocator(self, thing=None):
        clusto.flush()

        counter = clusto.Counter.get(self.entity, 'next', default=self.next)

        num = str(counter.value)

        if self.leadingZeros:
            num = num.rjust(self.digits, '0')

        if len(num) > self.digits:
            raise SimpleNameManagerException("Out of digits for the integer. "
                                             "Max of %d digits and we're at "
                                             "number %s." % (self.digits, num))
        
        nextname = self.basename + num

        counter.next()

        return (nextname, True)
        

class SimpleEntityNameManager(SimpleNameManager):    

    _driver_name = "simpleentitynamemanager"

    _record_allocations = False


    def allocate(self, clustotype, resource=None, number=True):
        """allocates a resource element to the given thing.

        resource - is passed as an argument it will be checked 
                   before assignment.  

        refattr - the attribute name on the entity that will refer back
                  this resource manager.

        returns the resource that was either passed in and processed 
        or generated.
        """

        if not isinstance(clustotype, type):
            raise TypeError("thing is not a Driver class")

        try:
            clusto.begin_transaction()

            if not resource:
                name, num = self.allocator()

                newobj = clustotype(name)

            else:
                name = resource
                newobj = clustotype(resource)


            super(SimpleEntityNameManager, self).allocate(newobj, name)

            clusto.commit()
        except Exception, x:
            clusto.rollback_transaction()
            raise
        
        return newobj


    def deallocate(self, thing, resource=None, number=True):
        raise Exception("can't deallocate an entity name, delete the entity instead.")




#!/usr/bin/env python
# -*- mode: python; sh-basic-offset: 4; indent-tabs-mode: nil; coding: utf-8 -*-
# vim: tabstop=4 softtabstop=4 expandtab shiftwidth=4 fileencoding=utf-8

import argparse
import sys

import clusto
from clusto import drivers
from clusto import script_helper


class Pool(script_helper.Script):
    '''
    Operate on a pool or on its objects. You can create, delete or show
    a pool and you can insert or remove items to/from a pool
    '''

    def __init__(self):
        script_helper.Script.__init__(self)

    def _get_pool(self, pool):
        p = clusto.get_by_name(pool)
        self.debug('%s is %s' % (pool, p))
        if not p or not isinstance(p, clusto.drivers.pool.Pool):
            self.error('The pool "%s" does not exist or is not a pool')
            return False
        return p

    def run_show(self, args):
        '''
        Prints the pool contents to stdout
        '''

        p = self._get_pool(args.pool[0])
        if not p:
            return -1
        print 'Printing the contents of pool %s' % p.name
        for item in p.contents():
            print '%s (%s)' % (item.name, item.type)

    def run_create(self, args):
        '''
        Creates a new, empty pool
        '''

        try:
            p = drivers.Pool(args.pool[0])
            return 0
        except Exception, e:
            self.warn(e)
            return 2

    def run_delete(self, args):
        '''
        Deletes a pool
        '''

        p = self._get_pool(args.pool[0])
        if not p:
            return -1
        sys.stdout.write('Are you sure you want to delete the pool %s (yes/NO)? ' % p.name)
        answer = sys.stdin.readline()
        if answer.strip() == 'yes':
            try:
                clusto.delete_entity(p.entity)
                return 0
            except Exception, e:
                self.error(e)
                return 3
        else:
            sys.stderr.write('Aborted\n')
            return 0

    def run_insert(self, args):
        '''
        Insert objects to a given pool
        '''

        if not args.objects:
            self.error('Please provide objects to insert to the pool')
            return -2
        p = self._get_pool(args.pool[0])
        if not p:
            return -1
        for obj in args.objects:
            try:
                obj = clusto.get_by_name(obj)
                self.debug(obj)
            except LookupError:
                self.warn('%s does not exist' % obj)
                continue
            if obj in p.contents():
                self.warn('%s is already in %s' % (obj.name, p.name))
                continue
            p.insert(obj)
        return 0

    def run_remove(self, args):
        '''
        Removes objects from a given pool
        '''

        if not args.objects:
            self.error('Please provide objects to remove from the pool')
            return -2
        p = self._get_pool(args.pool[0])
        if not p:
            return -1
        for obj in args.objects:
            try:
                obj = clusto.get_by_name(obj)
                self.debug(obj)
            except LookupError:
                self.warn('%s does not exist' % obj)
                continue
            if obj not in p.contents():
                self.warn('%s is not in %s' % (obj.name, p.name))
                continue
            p.remove(obj)
        return 0

    def run(self, args):
        getattr(self, 'run_%s' % args.action[0])(args)

    def _add_arguments(self, parser):
        parser.add_argument('action', nargs=1, metavar='action',
            choices=['show', 'create', 'delete', 'insert', 'remove'],
            help='Action to execute (show, create, delete, insert, remove)')
        parser.add_argument('pool', nargs=1, metavar='pool',
            help='Pool to modify (required)')
        parser.add_argument('objects', nargs='*',
            help='Objects to insert/remove (required if action = insert/remove)')


def main():
    pool, args = script_helper.init_arguments(Pool)
    return(pool.run(args))

if __name__ == '__main__':
    sys.exit(main())




#!/usr/bin/env python
from socket import socket, AF_INET, SOCK_DGRAM
from traceback import format_exc
from time import strftime, time, localtime, sleep
from struct import unpack
import sys

import logging

from clusto.services.config import conf, get_logger
log = get_logger('clusto.snmp', 'INFO')

import logging
runtime = logging.getLogger('scapy.runtime')
runtime.setLevel(logging.ERROR)
loading = logging.getLogger('scapy.loading')
loading.setLevel(logging.ERROR)

from scapy.all import SNMP

from clusto.drivers import IPManager, PenguinServer
import clusto

sys.path.insert(0, '/var/lib/clusto')
import rackfactory

def update_clusto(trap):
    ts = strftime('[%Y-%m-%d %H:%M:%S]')
    if trap['operation'] != 1:
        return

    if not trap['mac'].startswith('00'):
        return

    switch = IPManager.get_devices(trap['switch'])
    if not switch:
        log.warning('Unknown trap source: %s' % trap['switch'])
        return
    else:
        switch = switch[0]

    if not switch.attrs(key='snmp', subkey='discovery', value=1, merge_container_attrs=True):
        log.debug('key=snmp, subkey=discovery for %s not set to 1, ignoring trap' % switch.name)
        return

    rack = switch.parents(clusto_types=['rack'])[0]

    try:
        factory = rackfactory.get_factory(rack.name)
        if not factory:
            log.warning('Unable to locate rack factory for %s' % rack.name)
            return
    except:
        log.error(format_exc())
        return

    server = switch.get_connected('nic-eth', trap['port'])
    if not server:
        servernames = clusto.get_by_name('servernames')
        clusto.SESSION.clusto_description = 'SNMP allocate new server'
        driver = factory.get_driver(trap['port'])
        server = servernames.allocate(driver)
        log.info('Created new %s on %s port %s: %s' % (driver.__name__, trap['switch'], trap['port'], server.name))

    try:
        clusto.begin_transaction()
        if not trap['mac'] in server.attr_values(key='bootstrap', subkey='mac', value=trap['mac']):
            log.debug('Adding bootstrap mac to', server.name)
            server.add_attr(key='bootstrap', subkey='mac', value=trap['mac'])

        factory.add_server(server, trap['port'])
        switch.set_port_attr('nic-eth', trap['port'], 'vlan', trap['vlan'])

        clusto.SESSION.clusto_description = 'SNMP update MAC and connections on %s' % server.name
        clusto.commit()
    except:
        log.error(format_exc())
        clusto.rollback_transaction()

    log.debug(repr(trap))

def trap_listen():
    sock = socket(AF_INET, SOCK_DGRAM)
    sock.bind(('0.0.0.0', 162))
    while True:
        data, address = sock.recvfrom(4096)
        trap = SNMP(data)

        for var in trap.PDU.varbindlist:
            if var.oid.val == '1.3.6.1.4.1.9.9.215.1.1.8.1.2.1':
                var.value.val = var.value.val[:11]
                operation, vlan, mac, port = unpack('>cH6sH', var.value.val)
                result = {
                    'operation': ord(operation),
                    'vlan': vlan,
                    'mac': ':'.join([('%x' % ord(x)).rjust(2, '0') for x in mac]).lower(),
                    'port': port,
                    'switch': address[0],
                }
                yield result



#!/usr/bin/env python
# -*- mode: python; sh-basic-offset: 4; indent-tabs-mode: nil; coding: utf-8 -*-
# vim: tabstop=4 softtabstop=4 expandtab shiftwidth=4 fileencoding=utf-8
#
# Shell command
# Copyright 2009, Ron Gorodetzky <ron@fflick.com>

import argparse
import sys

import IPython

import clusto
from clusto import script_helper
from clusto import *


class Shell(script_helper.Script):
    '''
    This is a powerful, interactive clusto shell. The full clusto API
    is available in python idioms.

    Use at your own risk
    '''

    def __init__(self):
        script_helper.Script.__init__(self)

    def run(self, args):
        banner = None
        if IPython.__version__ >= '0.11':
            config=IPython.config.application.Config()
        if not sys.stdin.isatty() or args.files:
            opts = [
                '-noautoindent',
                '-nobanner',
                '-colors', 'NoColor',
                '-noconfirm_exit',
                '-nomessages',
                '-nosep',
                '-prompt_in1', '\x00',
                '-prompt_in2', '\x00',
                '-prompt_out', '\x00',
                '-xmode', 'Plain',
            ]
            if IPython.__version__ >= '0.11':
                config.TerminalInteractiveShell.autoindent = False
                config.TerminalIPythonApp.display_banner = False
                config.TerminalInteractiveShell.color_info = False
                config.TerminalInteractiveShell.confirm_exit = False
                config.TerminalInteractiveShell.quiet = True
                config.TerminalIPythonApp.ignore_old_config = True
                config.TerminalInteractiveShell.separate_in = ''
                config.PromptManager.in_template = '\x00'
                config.PromptManager.in_template1 = '\x00'
                config.PromptManager.in_template2 = '\x00'
                config.PromptManager.out_template = '\x00'
                config.TerminalInteractiveShell.xmode = 'Plain'
        else:
            opts = [
                '-prompt_in1', 'clusto [\#]> ',
                '-prompt_out', 'out [\#]> ',
            ]
            banner = '\nThis is the clusto shell. Respect it.'
            if IPython.__version__ >= '0.11':
                config.PromptManager.in_template = 'clusto [\#]> '
                config.PromptManager.out_template = 'out [\#]> '
        if args.loglevel == 'DEBUG':
            opts.append('-debug')
            config.debug = True

        plugins = script_helper.load_plugins(self.config)
        plugins.update(globals())
        if IPython.__version__ < '0.11':
            from IPython.Shell import IPShellEmbed
            ipshell = IPShellEmbed(opts, user_ns=plugins)
            if banner:
                ipshell.set_banner(banner)
        else:
            from IPython.frontend.terminal import embed
            ipshell = embed.InteractiveShellEmbed(banner1=banner,
                config=config, user_ns=plugins)
        ipshell()

    def _add_arguments(self, parser):
        parser.add_argument('files', nargs='?',
            help='Files to load & run')

    def add_subparser(self, subparsers):
        parser = self._setup_subparser(subparsers)
        self._add_arguments(parser)

def main():
    shell, args = script_helper.init_arguments(Shell)
    return(shell.run(args))

if __name__ == '__main__':
    sys.exit(main())




"""
Test the basic Driver object
"""

import unittest
from clusto.test import testbase
import datetime

import clusto
from clusto import Attribute
from clusto.drivers.base import *
from clusto.drivers import Pool
from clusto.exceptions import *

class TestDriverAttributes(testbase.ClustoTestBase):

    def testSetAttrs(self):

        d1 = Driver('d1')
        d1.set_attr('foo', 'bar')

        self.assertEqual(d1.attr_items(),
                         [(('foo', None, None), 'bar')])

        d1.set_attr('foo', 'bar2')
        self.assertEqual(d1.attr_items(),
                         [(('foo', None, None), 'bar2')])

        d1.add_attr('foo', 'bar3')

        self.assertEqual(sorted(d1.attr_items()),
                         sorted(
                         [(('foo', None, None), 'bar2'),
                          (('foo', None, None), 'bar3')]))

        self.assertRaises(DriverException, d1.set_attr, 'foo', 'bar4')


        d2 = Driver('d2')
        d2.add_attr('a', number=0, subkey='foo', value='bar1')
        d2.add_attr('a', number=1, subkey='foo', value='bar1')
        d2.add_attr('a', number=2, subkey='foo', value='bar1')

        d2.set_attr('a', 't1')

        self.assertEqual(sorted(d2.attr_items()),
                         sorted([(('a', 0, 'foo'), 'bar1'),
                                 (('a', 1, 'foo'), 'bar1'),
                                 (('a', 2, 'foo'), 'bar1'),
                                 (('a', None, None), 't1'),]))

    def testGettingAttrs(self):

        d1 = Driver('d1')

        d1.add_attr('foo', 'bar')
        d1.add_attr('foo', 'bar1', number=0)

        self.assertEqual(sorted(d1.attr_items()),
                         [(('foo', None, None), 'bar'), 
                          (('foo', 0, None), 'bar1')])



        self.assertEqual(d1.attr_items(number=True),
                         [(('foo', 0, None), 'bar1')])

    def testGettingAttrValues(self):
        d1 = Driver('d1')
        d2 = Driver('d2')
        
        d1.add_attr('foo', 'bar')
        d1.add_attr('foo0', 'bar1')
        d2.add_attr('d1', d1)

        clusto.flush()

        self.assertEqual(sorted(['bar', 'bar1']),
                         sorted(d1.attr_values('foo.*', regex=True)))

        self.assertEqual([d1], d2.attr_values())
        

    def testGettingAttrsMultipleTimes(self):
        d1 = Driver('d1')
        d2 = Driver('d2')
        
        d1.add_attr('foo', 'bar')
        d1.add_attr('foo0', 'bar1')
        d2.add_attr('d1', d1)

        clusto.flush()

        d = clusto.get_by_name('d1')
        
        self.assertEqual(len(d.references()), 1)
        self.assertEqual(len(d.attrs()), 2)


        
        
    def testNumberedAttrs(self):

        d1 = Driver('d1')

        d1.add_attr('foo', 'bar')

        d1.add_attr('foo', 'bar1', number=5)
        d1.add_attr('foo', 'bar2', number=6)

        clusto.flush()

        self.assertEqual(sorted(d1.attr_items()),
                         sorted([(('foo', None, None), 'bar'), 
                          (('foo', 5, None), 'bar1'), 
                          (('foo', 6, None), 'bar2')]))

        self.assertEqual(sorted(d1.attr_items(number=True)),
                         sorted([(('foo', 5, None), 'bar1'), 
                          (('foo', 6, None), 'bar2')]))


    def testAutoNumberedAttrs(self):
        d1 = Driver('d1')

        d1.add_attr('foo', 'bar')

        d1.add_attr('foo', 'bar1', number=True)
        d1.add_attr('foo', 'bar2', number=True)

        clusto.flush()

        self.assertEqual(sorted(d1.attr_items()),
                         sorted([(('foo', None, None), 'bar'),
                                 (('foo', 0, None), 'bar1'),
                                 (('foo', 1, None), 'bar2')]))

        self.assertEqual(sorted(d1.attr_items(number=True)),
                         sorted([(('foo', 0, None), 'bar1'),
                                 (('foo', 1, None), 'bar2')]))

        
    def testSubKeyAttrs(self):

        d1 = Driver('d1')

        d1.add_attr('foo', 'bar', subkey='subfoo')
        d1.add_attr('foo', 'caz', subkey='subbar')

        self.assertEqual(sorted(d1.attr_key_tuples()),
                         sorted([('foo',None,'subfoo'), ('foo',None,'subbar')]))

    def testNumberedAttrsWithSubKeys(self):

        d1 = Driver('d1')

        d1.add_attr(key='foo', value='bar1', number=True, subkey='one')
        d1.add_attr(key='foo', value='bar2', number=True, subkey='two')
        
        self.assertEqual(d1.attr_items(),
                         [(('foo', 0, 'one'), 'bar1'),
                          (('foo', 1, 'two'), 'bar2')])

    def testGettingSpecificNumberedAttrs(self):
        
        d1 = Driver('d1')

        d1.add_attr(key='foo', value='bar1', number=True, subkey='one')
        d1.add_attr(key='foo', value='bar2', number=True, subkey='two')
        d1.add_attr(key='foo', value='bar3', number=True, subkey='three')
        d1.add_attr(key='foo', value='bar4', number=True, subkey='four')

        self.assertEqual(list(d1.attr_items(key='foo', number=2)),
                         [(('foo',2,'three'), 'bar3')])
        
        self.assertEqual(list(d1.attr_items(key='foo', number=0)),
                         [(('foo',0,'one'), 'bar1')])
        
    def testGettingAttrsWithSpecificValues(self):

        d1 = Driver('d1')

        d1.add_attr(key='foo', value='bar1', number=True, subkey='one')
        d1.add_attr(key='foo', value='bar2', number=True, subkey='two')
        d1.add_attr(key='foo', value='bar3', number=True, subkey='three')
        d1.add_attr(key='foo', value='bar4', number=True, subkey='four')

        self.assertEqual(list(d1.attr_items(value='bar3')),
                         [(('foo',2,'three'), 'bar3')])
        
        self.assertEqual(list(d1.attr_items(value='bar1')),
                         [(('foo',0,'one'), 'bar1')])
        

                          
    def testDelAttrs(self):
        d1 = Driver('d1')

        d1.add_attr(key='foo', value='bar1', number=True, subkey='one')
        d1.add_attr(key='foo', value='bar2', number=True, subkey='two')
        d1.add_attr(key='foo', value='bar3', number=True, subkey='three')
        d1.add_attr(key='foo', value='bar4', number=True, subkey='four')

        d1.del_attrs(key='foo', value='bar4')

        
        self.assertEqual(list(d1.attr_items(value='bar4')),
                         [])

        self.assertEqual(list(d1.attr_items(value='bar3')),
                         [(('foo',2,'three'), 'bar3')])

        d1.del_attrs(key='foo', subkey='three', number=2)
        self.assertEqual(list(d1.attr_items(value='bar3')),
                         [])


    def testHasAttr(self):
        
        d1 = Driver('d1')

        d1.add_attr(key='foo', value='bar1', number=True, subkey='one')
        d1.add_attr(key='foo', value='bar2', number=True, subkey='two')
        d1.add_attr(key='foo', value='bar3', number=True, subkey='three')
        d1.add_attr(key='foo', value='bar4', number=True, subkey='four')

        self.assertFalse(d1.has_attr(key='foo', number=False))
        self.assertTrue(d1.has_attr(key='foo', number=True))
        self.assertTrue(d1.has_attr(key='foo', number=1, subkey='two'))

    def testHiddenAttrs(self):

        d1 = Driver('d1')

        d1.add_attr(key='foo', value='bar1', number=True, subkey='one')
        d1.add_attr(key='foo', value='bar2', number=True, subkey='two')
        d1.add_attr(key='_foo', value='bar3', number=True, subkey='three')
        d1.add_attr(key='_foo', value='bar4', number=True, subkey='four')

        self.assertEqual(d1.attr_items(ignore_hidden=True),
                         [(('foo',0,'one'), 'bar1'), (('foo',1,'two'), 'bar2')])


    def testAttributeGetValueAfterAdd(self):

        d1 = Driver('d1')

        d1.add_attr('foo', 2)
        self.assertEqual(d1.attr_items('foo'), [(('foo',None,None), 2)])
        d1.add_attr('bar', 3)
        self.assertEqual(d1.attr_items('foo'), [(('foo',None,None), 2)])
        self.assertEqual(d1.attr_items('bar'), [(('bar',None,None), 3)])


    def testGetByAttr(self):

        d1 = Driver('d1')
        d1.add_attr('foo', 1)

        d2 = Driver('d2')
        d2.add_attr('foo', 2)

        d3 = Driver('d3')
        d3.add_attr('bar', 3)

        clusto.flush()

        result = Driver.get_by_attr('foo', 2)

        self.assertEqual(result, [d2])
        
    def testAttrCount(self):
        
        d1 = Driver('d1')

        d1.add_attr(key='foo', value='bar1', number=True, subkey='one')
        d1.add_attr(key='foo', value='bar2', number=True, subkey='two')
        d1.add_attr(key='foo', value='bar3', number=True, subkey='three')
        d1.add_attr(key='foo', value='bar4', number=True, subkey='four')
        
        self.assertEqual(d1.attr_query(key='foo', number=2, count=True), 1)
        
        self.assertEqual(d1.attr_query(key='foo', number=0, count=True), 1)

        self.assertEqual(d1.attr_query(key='foo', number=False, count=True), 0)
        self.assertEqual(d1.attr_query(key='foo', count=True), 4)

        self.assertEqual(d1.attr_query(subkey='four', count=True), 1)


        d1.del_attrs(key='foo', value='bar1', number=True, subkey='one')
        d1.add_attr(key='foo', value='bar5', number=True, subkey='five')
        self.assertEqual(d1.attr_query(key='foo', number=0, count=True), 0)
        self.assertEqual(d1.attr_query(key='foo', number=4, count=True), 1)
        
    def testSetAttrAlreadySet(self):

        d1 = Driver('d1')

        version = clusto.get_latest_version_number()
        
        d1.set_attr(key='foo', value='bar1')

        self.assertEqual(version+1, clusto.get_latest_version_number())

        d1.set_attr(key='foo', value='bar1')

        self.assertEqual(version+1, clusto.get_latest_version_number())

class TestDriverContainerFunctions(testbase.ClustoTestBase):
    
    def testInsert(self):

        d1 = Driver('d1')
        d2 = Driver('d2')

        d1.insert(d2)
        
        clusto.flush()

        d = clusto.get_by_name('d1')

        self.assertEqual(d.attr_items(ignore_hidden=False),
                         [(('_contains', 0, None), d2)])

    def testRemove(self):
        
        d1 = Driver('d1')
        d2 = Driver('d2')

        d1.insert(d2)
        
        clusto.flush()

        d = clusto.get_by_name('d1')
        d.remove(d2)

        clusto.flush()

        self.assertEqual(d.attr_items(ignore_hidden=False),
                         [])

    def testContents(self):
        
        d1 = Driver('d1')
        d2 = Driver('d2')

        d1.insert(d2)
        
        self.assertEqual(d1.contents(), [d2])
                         

    def testChildrenContents(self):

        p1 = Pool('p1')
        p2 = Pool('p2')

        d1 = Driver('d1')
        d2 = Driver('d2')

        p1.insert(d1)
        p2.insert(d2)
        p2.insert(p1)

        self.assertEqual(sorted([p1,d1,d2]),
                         sorted(p2.contents(search_children=True)))
        
    def testMultipleInserts(self):

        d1 = Driver('d1')
        d2 = Driver('d2')
        d3 = Driver('d3')

        d1.insert(d2)
        
        self.assertRaises(TypeError, d3.insert, d2)

    def testNumberedInserts(self):

        d1 = Driver('d1')

        d1.insert(Driver('d2'))
        d1.insert(Driver('d3'))
        d1.insert(Driver('d4'))
        d1.insert(Driver('d5'))
        d1.insert(Driver('d6'))

        
        self.assertEqual(range(5),
                         [x.number for x in d1.attrs(ignore_hidden=False)])
        

    def testParents(self):

        class OtherDriver(Driver):
            _clusto_type = 'otherdriver'
            _driver_name = 'otherdriver'


        p1 = Pool('toppool')
        d1 = Pool('grandparent')
        d1a = Pool('othergrandparent')
        d1b = OtherDriver('anothergrandparent')
        d2 = Driver('parent')
        d3 = Driver('child')

        d1b.add_attr('foo', 'someval')
        p1.insert(d1a)
        d1b.insert(d2)
        d1a.insert(d2)
        d1.insert(d2)
        d2.insert(d3)

        self.assertEqual(sorted([d1,d1a, p1]),
                         sorted(d3.parents(clusto_types=[Pool], search_parents=True)))

        self.assertEqual([d2],
                         d3.parents())

        self.assertEqual(['someval'],
                         d3.attr_values('foo', merge_container_attrs=True))

class TestDriver(testbase.ClustoTestBase):
    
    def testCreatingDriverWithUsedName(self):
        
        d1 = Driver('d1')

        self.assertRaises(NameException, Driver, 'd1')

        d1.attrs()

    def testDriverSets(self):
        
        d1 = Driver('d1')
        d2 = Driver('d2')

        s = set([d1,d1,d2])

        self.assertEquals(len(s), 2)

class ATestDriver(Driver):

    _clusto_type = "tester"
    _driver_name = "testdriver"

    _properties = {'propA': None,
                   'propB': 'foo',
                   'propC': 5 }

class TestDriverProperties(testbase.ClustoTestBase):
    
    def testPropDefaultGetter(self):

        d = ATestDriver('d')

        self.assertEqual(None, d.propA)
        self.assertEqual('foo', d.propB)
        self.assertEqual(5, d.propC)

    def testPropSetter(self):

        d = ATestDriver('d')

        self.assertEqual(None, d.propA)

        d.propA = 'foo'
        self.assertEqual('foo', d.propA)

        d.propA = 'bar'
        self.assertEqual('bar', d.propA)

        d.propA = 10
        self.assertEqual(10, d.propA)

    def testPropSetterMultipleObjects(self):
        d = ATestDriver('d')
        d2 = ATestDriver('d2')

        d.propB = 'bar'
        d2.propB = 'cat'

        self.assertEqual(d2.propB, 'cat')
        self.assertEqual(d.propB, 'bar')

class TestDriverQueries(testbase.ClustoTestBase):
    
    def data(self):

        d1 = Driver('d1')
        d2 = Driver('d2')
        d3 = Driver('d3')

        d1.add_attr('_foo', 'bar1')
        d1.add_attr('car', 'baz')
        d1.add_attr('car', 'baz')
        d1.add_attr('d', 'dee', number=True)
        d1.add_attr('d', 'dee', number=True)
        d1.add_attr('a', 1)
        d1.add_attr('a', 1, subkey='t')
        d1.add_attr('a', 1, subkey='g')
        d1.add_attr('a', 1, subkey='z', number=4)
        d1.add_attr('a', 1, subkey='z', number=5)
        d1.add_attr('a', 1, subkey='z', number=6)
        
        d1.set_attr('d2', d2)
        d1.set_attr('d3', d3)

        d2.set_attr('aaa', 1)
        d2.set_attr('aab', 2)
        d2.set_attr('aac', 3)



    def testAttrAndQueryEqual(self):

        d1 = clusto.get_by_name('d1')
        d2 = clusto.get_by_name('d2')
        d3 = clusto.get_by_name('d3')

        self.assertEqual(d1.attrs('a'), d1.attr_query('a'))

        self.assertEqual(d1.attrs('a', 1), d1.attr_query('a', 1))

        self.assertEqual(d1.attrs('a', 1, number=True), 
                         d1.attr_query('a', 1, number=True))

        self.assertEqual(d1.attrs('a', 1, number=5), 
                         d1.attr_query('a', 1, number=5))

        self.assertEqual(d1.attrs(value='dee'), 
                         d1.attr_query(value='dee'))


        self.assertEqual(d1.attrs(value='_foo'), 
                         d1.attr_query(value='_foo'))

        self.assertEqual(d1.attrs(key='_foo'), 
                         d1.attr_query(key='_foo'))

        self.assertEqual(d1.attrs(key='a', subkey=None), 
                         d1.attr_query(key='a', subkey=None))

        self.assertEqual(d1.attrs(value=d2), 
                         d1.attr_query(value=d2))


        self.assertEqual(d1.attrs(subkey='z'),
                         d1.attr_query(subkey='z'))


    def testDoAttrQuery(self):

        d1 = clusto.get_by_name('d1')
        d2 = clusto.get_by_name('d2')
        self.assertEqual(set(Driver.get_by_attr(key='a*', glob=True)),
                         set([d1,d2]))



import sys
import os

sys.path.insert(0, os.curdir)


import unittest

import clusto

import ConfigParser

DB='sqlite:///:memory:'
ECHO=False

class ClustoTestResult(unittest.TestResult):
    def addError(self, test, err):
        """Called when an error has occurred. 'err' is a tuple of values as
        returned by sys.exc_info().
        """
        print >>sys.stderr, "ERROR HERE!"
        clusto.rollback_transaction()
        self.errors.append((test, self._exc_info_to_string(err, test)))
        


class ClustoTestBase(unittest.TestCase):



    def data(self):
        pass
    
    def setUp(self):

        conf = ConfigParser.ConfigParser()
        conf.add_section('clusto')
        conf.set('clusto', 'dsn', DB)
        clusto.connect(conf,echo=ECHO)
        clusto.clear()
        clusto.SESSION.close()
        clusto.init_clusto()
        self.data()


    def tearDown(self):
        if clusto.SESSION.is_active:
            raise Exception("SESSION IS STILL ACTIVE in %s" % str(self.__class__))

        clusto.clear()
        clusto.disconnect()
        clusto.METADATA.drop_all(clusto.SESSION.bind)



    def defaultTestResult(self):
        if not hasattr(self._testresult):
            self._testresult = ClustoTestResult()

        return self._testresult





from clusto.test import testbase

from clusto.drivers import BasicServer, BasicRack, IPManager
from clusto.drivers import BasicNetworkSwitch, BasicPowerStrip
from clusto.exceptions import ConnectionException
import clusto

class ServerInstallationTest(testbase.ClustoTestBase):
    """Test installing a server 

    Put the server into a rack 
    connect the server to a powerstrip and a networkswitch
    """

    def data(self):
        
        r1 = BasicRack('r1')
        
        sw1 = BasicNetworkSwitch('sw1')

        s1 = BasicServer('s1')
        
        p1 = BasicPowerStrip('p1')

        r1.insert(p1, (10,11))
        r1.insert(sw1, 12)
        r1.insert(s1, 1)

    def testServerRackLocation(self):

        r = clusto.get_by_name('r1')
        s = clusto.get_by_name('s1')
        
        self.assertEqual(BasicRack.get_rack_and_u(s)['RU'], [1])

        self.assertEqual(r.get_device_in(12),
                         clusto.get_by_name('sw1'))

        self.assertEqual(r.get_device_in(10),
                         clusto.get_by_name('p1'))

        self.assertEqual(r.get_device_in(11),
                         clusto.get_by_name('p1'))
        
        

    def testPortConnections(self):

        s = clusto.get_by_name('s1')
        sw = clusto.get_by_name('sw1')
        p1 = clusto.get_by_name('p1')

        sw.connect_ports('nic-eth', 1, s, 1)
        
        
        self.assertRaises(ConnectionException,
                          s.connect_ports, 'nic-eth', 1, sw, 2)

        p1.connect_ports(porttype='pwr-nema-5',
                        srcportnum=1,
                        dstdev=s,
                        dstportnum=1)
                        
        self.assertEqual(s.get_connected('pwr-nema-5', 1),
                         p1)


    def testSettingUpServer(self):
        
        from clusto.drivers import SimpleEntityNameManager

        servernames = SimpleEntityNameManager('servernames',
                                              basename='server',
                                              digits=4
                                              )

        newserver = servernames.allocate(BasicServer)
        

        sw = clusto.get_by_name('sw1')
        p1 = clusto.get_by_name('p1')
        r = clusto.get_by_name('r1')

        self.assertEqual('server0001', newserver.name)


        self.assertRaises(TypeError, r.insert, newserver, 1)

        r.insert(newserver,2)
        p1.connect_ports('pwr-nema-5', 1, newserver, 1)
        sw.connect_ports('nic-eth', 1, newserver, 1)
        sw.connect_ports('nic-eth', 3, p1, 1)

        self.assertEqual(BasicRack.get_rack_and_u(newserver)['rack'], r)

        ipman = IPManager('subnet-10.0.0.1', netmask='255.255.255.0', baseip='10.0.0.1')

        newserver.bind_ip_to_osport('10.0.0.2', 'eth0', porttype='nic-eth', portnum=1)

        ipvals = newserver.attrs(value='10.0.0.2')
        self.assertEqual(len(ipvals), 1)

        self.assertEqual(ipvals[0].value, '10.0.0.2')

        self.assertEqual(clusto.get_by_attr('ip', '10.0.0.2'), [newserver])


        aserver = servernames.allocate(BasicServer)

        ipattr = ipman.allocate(aserver)
        
        aserver.bind_ip_to_osport(ipattr.value, 'eth0', porttype='nic-eth', portnum=1)

        ip = aserver.attr_values(ipattr.key, number=ipattr.number, subkey=ipattr.subkey)

        self.assertEqual(aserver.get_port_attr('nic-eth', 1, 'osportname'),
                         'eth0')

        self.assertEqual(len(aserver.attrs(subkey='osportname', value='eth0')),
                         2)

        self.assertEqual(aserver.attrs(IPManager._attr_name,
                                       subkey='ipstring',
                                       number=aserver.attrs(IPManager._attr_name,
                                                            subkey='osportname',
                                                            value='eth0')[0].number,
                                       )[0].value,
                         '10.0.0.1')

        ipattr2 = ipman.allocate(aserver)

        self.assertEqual(sorted(aserver.get_ips()),
                         sorted(['10.0.0.1', '10.0.0.3']))
        
        
        
        
        

        



# -*- coding: utf-8 -*-
#
# © 2012 Paul D. Lathrop
# Author: Paul Lathrop <paul@tertiusfamily.net>
#

"""Clusto driver to represent a cage in a datacenter."""

from clusto.drivers.base import Location

class BasicCage(Location):
    """
    Basic Cage driver.
    """

    _clusto_type = "cage"
    _driver_name = "basiccage"



#!/usr/bin/env python
# -*- mode: python; sh-basic-offset: 4; indent-tabs-mode: nil; coding: utf-8 -*-
# vim: tabstop=4 softtabstop=4 expandtab shiftwidth=4 fileencoding=utf-8
#
# Clusto cluster management tool
# Copyright 2009, Ron Gorodetzky ron@parktree.net

import os
import setuptools
import sys


srcdir = os.path.join(os.path.dirname(sys.argv[0]), 'src')

setuptools.setup(
    name = "clusto",
    version = "0.7.3",
    packages = setuptools.find_packages('src'),
    author = "Ron Gorodetzky",
    author_email = "ron@parktree.net",
    license = 'BSD',
    description = "Cluster management and inventory system",
    install_requires = [
        'distribute',
        'argparse',
        'sqlalchemy>=0.7',
        'IPython>=0.10',
        'IPy',
    ],
    entry_points = {
        'console_scripts': [
            'clusto = clusto.script_helper:main',
            'clusto-allocate = clusto.commands.allocate:main',
            'clusto-attr = clusto.commands.attr:main',
            'clusto-console = clusto.commands.console:main',
            'clusto-deallocate = clusto.commands.deallocate:main',
            'clusto-info = clusto.commands.info:main',
            'clusto-list-pool = clusto.commands.list_pool:main',
            'clusto-pool = clusto.commands.pool:main',
            'clusto-reboot = clusto.commands.reboot:main',
            'clusto-rename = clusto.commands.rename:main',
            'clusto-initdb = clusto.commands.initdb:main',
            'clusto-shell = clusto.commands.shell:main',
            'clusto-list-all = clusto.commands.list_all:main',
        ],
    },
    zip_safe = False,
    package_dir = { '': 'src' },
    scripts = [
        os.path.join(srcdir, 'scripts', 'clusto-tree'),
        os.path.join(srcdir, 'scripts', 'clusto-update-info'),
        os.path.join(srcdir, 'scripts', 'clusto-update-db'),
        os.path.join(srcdir, 'scripts', 'clusto-dhcpd'),
        os.path.join(srcdir, 'scripts', 'clusto-httpd'),
      ],
      test_suite = "clusto.test.alltests.gettests",
)




import clusto
from clusto.test import testbase 
import itertools

from clusto.drivers import *

from clusto.drivers.resourcemanagers.simplenamemanager import SimpleNameManagerException


class ResourceManagerTests(testbase.ClustoTestBase):

    def testAllocate(self):

        rm = ResourceManager('test')
        d = Driver('d')

        rm.allocate(d, 'foo')

        self.assertEqual(rm.owners('foo'), [d])

    def testResourceCount(self):

        rm = ResourceManager('test')
        d = Driver('d')
        
        rm.allocate(d, 'foo')
        rm.allocate(d, 'bar')
        
        self.assertEqual(rm.count, 2)

    def testDeallocate(self):

        rm = ResourceManager('test')
        d = Driver('d')

        rm.allocate(d, 'foo')
        self.assertEqual(rm.count, 1)

        rm.deallocate(d, 'foo')
        self.assertEqual(rm.count, 0)
        self.assertEqual(rm.owners('foo'), [])

    def testGeneralDeallocate(self):

        rm = ResourceManager('test')
        d = Driver('d')

        rm.allocate(d, 'foo')
        rm.allocate(d, 'bar')
        
        self.assertEqual(rm.count, 2)
        self.assertEqual(sorted([x.value for x in rm.resources(d)]),
                         sorted(['foo', 'bar']))

        rm.deallocate(d)

        self.assertEqual(rm.count, 0)
        self.assertEqual(sorted(rm.resources(d)),
                         sorted([]))


    def testResourceAttrs(self):

        
        rm = ResourceManager('test')
        d = Driver('d')

        rm.allocate(d, 'foo')
        rm.allocate(d, 'bar')

        rm.add_resource_attr(d, 'foo', 'attr1', 10)

        self.assertEqual(rm.get_resource_attr_values(d, 'foo', 'attr1'), [10])

        rm.add_resource_attr(d, 'foo', 'attr1', 20)

        self.assertEqual(sorted(rm.get_resource_attr_values(d, 'foo', 'attr1')),
                         sorted([10, 20]))

        rm.del_resource_attr(d, 'foo', 'attr1')
        self.assertEqual(rm.get_resource_attr_values(d, 'foo', 'attr1'), [])

        rm.set_resource_attr(d,'bar', 'attr2', 1)        
        self.assertEqual(rm.get_resource_attr_values(d, 'bar', 'attr2'), [1])

        rm.set_resource_attr(d,'bar', 'attr2', 2)
        self.assertEqual(rm.get_resource_attr_values(d, 'bar', 'attr2'), [2])

    def testReserveResource(self):

        rm = ResourceManager('test')
        d = Driver('d')

        rm.allocate(d, 'foo')

        rm.allocate(rm, 'bar')
        

        self.assertRaises(ResourceException, rm.allocate, d, 'bar')
        



"""
PortMixin is a basic mixin to be used with devices that have ports
"""

import re

import clusto


from clusto.exceptions import ConnectionException

class PortMixin:
    """Provide port capabilities to devices
    
    The ports are defined in the Driver's _portmeta dictionary:

    _portmeta = { '<porttype>' : {'numports': <num> }}

    Several ports types can be defined in this dictionary.  Currently
    'numports' is the only porttype attribute used.  This data does not get
    stored as Entity attributes in the clusto db.  They live only in the class
    definition.

    Port data gets stored in the DB as the connect to other ports.  The
    keynames are of the form '_port-<porttype>'.  Each port has a specific
    number associated with it (usually the same number as on the physical
    device itself) and can have several port attributes.  There are no
    restrictions on attributes but some common ones might be: osname,
    cabletype, status, etc.
    
    """
    
    # _portmeta = { 'porttype' : {'numports': 10 }}

    _portmeta = { 'pwr-nema-5' : { 'numports':1, },
                  'nic-eth' : { 'numports':2, },
                  }


    def _port_key(self, porttype):
        
        return 'port-' + porttype
    
    def _ensure_portnum(self, porttype, num):

        if not isinstance(num, int):
            raise TypeError("Port number '%s' needs to be an integer." % str(num))
        elif num < 1:
            raise TypeError("Port number '%s' needs to be greater than 0." % str(num))
        if not self._portmeta.has_key(porttype) \
                or num > self._portmeta[porttype]['numports']:
            raise ConnectionException("No port %s:%s exists on %s." % (porttype, str(num), self.name))

        return num

    def connect_ports(self, porttype, srcportnum, dstdev, dstportnum):
        """connect a local port to a port on another device
        """


        for dev, num in [(self, srcportnum), (dstdev, dstportnum)]:

            if not hasattr(dev, 'port_exists'):
                msg = "%s has no ports."
                raise ConnectionException(msg % (dev.name))

            num = dev._ensure_portnum(porttype, num)

            if not dev.port_exists(porttype, num):
                msg = "port %s:%d doesn't exist on %s"
                raise ConnectionException(msg % (porttype, num, dev.name))

        
            if not dev.port_free(porttype, num):
                msg = "port %s:%d on %s is already in use"
                raise ConnectionException(msg % (porttype, num, dev.name))

        try:
            clusto.begin_transaction()
            self.set_port_attr(porttype, srcportnum, 'connection', dstdev)
            self.set_port_attr(porttype, srcportnum, 'otherportnum', dstportnum)
            
            dstdev.set_port_attr(porttype, dstportnum, 'connection', self)
            dstdev.set_port_attr(porttype, dstportnum, 'otherportnum', srcportnum)
            clusto.commit()
        except Exception, x:
            clusto.rollback_transaction()
            raise x

    def disconnect_port(self, porttype, portnum):
        """disconnect both sides of a port"""

        portnum = self._ensure_portnum(porttype, portnum)

        if not self.port_free(porttype, portnum):

            dev = self.get_connected(porttype, portnum)
            
            otherportnum = self.get_port_attr(porttype, portnum, 'otherportnum')
            
            clusto.begin_transaction()
            try:
                dev.del_port_attr(porttype, otherportnum, 'connection')
                dev.del_port_attr(porttype, otherportnum, 'otherportnum')
                
                self.del_port_attr(porttype, portnum, 'connection')
                self.del_port_attr(porttype, portnum, 'otherportnum')
                clusto.commit()
            except Exception, x:
                clusto.rollback_transaction()
                raise x
            

    def get_connected(self, porttype, portnum):
        """return the device that the given porttype/portnum is connected to"""

        portnum = self._ensure_portnum(porttype, portnum)

        if not self.port_exists(porttype, portnum):
            msg = "port %s:%d doesn't exist on %s"
            raise ConnectionException(msg % (porttype, portnum, self.name))
            

        return self.get_port_attr(porttype, portnum, 'connection')
            

    def ports_connectable(self, porttype, srcportnum, dstdev, dstportnum):
        """test if the ports you're trying to connect are compatible.
        """

        return (self.port_exists(porttype, srcportnum) 
                and dstdev.port_exists(porttype, dstportnum))
 
    def port_exists(self, porttype, portnum):
        """return true if the given port exists on this device"""
        
        if ((porttype in self._portmeta)):
            try:
                portnum = self._ensure_portnum(porttype, portnum)
                return True
            except ConnectionException:
                return False
        else:
            return False

    def port_free(self, porttype, portnum):
        """return true if the given porttype and portnum are not in use"""
        
        portnum = self._ensure_portnum(porttype, portnum)

        if (not self.port_exists(porttype, portnum) or
            self.has_attr(key=self._port_key(porttype), number=portnum, 
                         subkey='connection')):
            return False
        else:
            return True
        

    def add_port_attr(self, porttype, portnum, key, value):
        """add an attribute on the given port"""

        portnum = self._ensure_portnum(porttype, portnum)

        self.add_attr(key=self._port_key(porttype),
                     number=portnum,
                     subkey=key,
                     value=value)

    def set_port_attr(self, porttype, portnum, key, value):
        """set an attribute on the given port"""

        portnum = self._ensure_portnum(porttype, portnum)

        self.set_attr(key=self._port_key(porttype),
                     number=portnum,
                     subkey=key,
                     value=value)


    def del_port_attr(self, porttype, portnum, key, value=()):
        """delete an attribute on the given port"""

        portnum = self._ensure_portnum(porttype, portnum)

        if value is ():
            self.del_attrs(key=self._port_key(porttype),
                          number=portnum,
                          subkey=key)
        else:

            self.del_attrs(key=self._port_key(porttype),
                          number=portnum,
                          subkey=key,
                          value=value)
            
                     
    def get_port_attr(self, porttype, portnum, key):
        """get an attribute on the given port"""

        portnum = self._ensure_portnum(porttype, portnum)

        attr = self.attrs(key=self._port_key(porttype),
                          number=portnum,
                          subkey=key)

        if len(attr) > 1:
            raise ConnectionException("Somehow more than one attribute named "
                                      "%s is associated with port %s:%d on %s"
                                      % (key, porttype, portnum, self.name))

        elif len(attr) == 1:
            return attr[0].value

        else:
            return None
            
    @property
    def port_info(self):
        """return a list of tuples containing port information for this device
        
        format:
            port_info[<porttype>][<portnum>][<portattr>]
        """

        portinfo = {}
        for ptype in self.port_types:
            portinfo[ptype]={}
            for n in range(1, self._portmeta[ptype]['numports'] + 1):
                portinfo[ptype][n] = {'connection': self.get_port_attr(ptype, n, 'connection'),
                                      'otherportnum': self.get_port_attr(ptype, n, 'otherportnum')}

        return portinfo

    @property
    def port_info_tuples(self):
        """return port information as a list of tuples that are suitble for use
        as *args to connect_ports

        format:
          [ ('porttype', portnum, <connected device>, <port connected to>), ... ]
        """
        
        t = []
        d = self.port_info
        for porttype, numdict in d.iteritems():
            for num, stats in numdict.iteritems():
                t.append((porttype, num, 
                          stats['connection'], stats['otherportnum']))
        
        return t

                         

    
    @property
    def free_ports(self):
        
        return [(pinfo[0], pinfo[1]) for pinfo in self.port_info_tuples if pinfo[3] == None]

    @property
    def connected_ports(self):
        """Return a list of connected ports"""

        pdict = {}
        for ptype in self.port_types:

            portlist = [a.number for a in self.attrs(self._port_key(ptype), 
                                                     subkey='connection')]
            portlist.sort()
            pdict[ptype] = portlist

        return pdict

    @property
    def port_types(self):
        return self._portmeta.keys()






import clusto
from clusto.schema import select, and_, ATTR_TABLE, Attribute, func, Counter
from clusto.drivers.base import Driver, ClustoMeta
from clusto.exceptions import ResourceTypeException, ResourceNotAvailableException, ResourceException



class ResourceManager(Driver):
    """The ResourceManager driver should be subclassed by a driver that will
    manage a resource such as IP allocation, MAC Address lists, etc.

    This base class just allocates unique integers.

    Resources are attributes on Entities that are managed by a ResourceManger.
    The implementation has the following properties:

    1. The Entity being assigned the resource gets an attribute who's key is
    defined by the resource manager, a number assigned by the resource manager
    (sometimes related to the resource being allocated), and a value which is
    a representation of the resource.

    2. The Entity gets an additional attribute who's key, and number match the
    allocated resource, but with subkey='manager', and value that is a
    reference to the resource manager assigning the resource.

    Any additional attributes with same attribute key and number are
    considered part of the resource and can be managed by the resource
    manager.
    """
    

    _clusto_type = "resourcemanager"
    _driver_name = "resourcemanager"

    _attr_name = "resource"
    _record_allocations = True
    


        
    def allocator(self, thing=None):
        """return an unused resource from this resource manager"""

        raise NotImplemented("No allocator implemented for %s you must explicitly specify a resource."
                             % self.name)


    def ensure_type(self, resource, number=True, thing=None):
        """checks the type of a given resourece

        if the resource is valid return it and optionally convert it to
        another format.  The format it returns has to be compatible with 
        attribute naming 
        """
        return (resource, number)


    def get_resource_number(self, thing, resource):
        """Get the number for a resource on a given entity."""
        
        resource, number = self.ensure_type(resource, thing=thing)
        
        attrs = thing.attrs(self._attr_name, value=resource)

        if attrs:
            return attrs[0].number
        else:
            raise ResourceException("%s isn't assigned resource %s"
                                    % (thing.name, str(resource)))


    def get_resource_attr_values(self, thing, resource, key, number=True):
        """Get the value for the attrs on the resource assigned to a given entity matching the given key."""
        
        return [x.value for x in self.get_resource_attrs(thing, resource,
                                                         key, number)]

    
    def get_resource_attrs(self, thing, resource, key=(), number=True):
        """Get the Attributes for the attrs on the resource assigned to a given enttiy matching the given key."""
        
        resource, number = self.ensure_type(resource, number, thing=thing)
        
        return thing.attrs(self._attr_name, number=number, subkey=key)

    
    def add_resource_attr(self, thing, resource, key, value, number=True):
        """Add an Attribute for the resource assigned to a given entity setting the given key and value"""
        
        resource, number = self.ensure_type(resource, number, thing=thing)

        attr = thing.add_attr(self._attr_name, number=number, subkey=key, value=value)
        return attr


    def set_resource_attr(self, thing, resource, key, value, number=True):
        """Set an Attribute for the resource assigned to a given entity with the given key and value"""
        
        resource, number = self.ensure_type(resource, number, thing=thing)
        attr = thing.set_attr(self._attr_name, number=number, subkey=key, value=value)

        return attr


    def del_resource_attr(self, thing, resource, key, value=(), number=True):
        """Delete an Attribute for the resource assigned to a given entity matching the given key and value"""
        
        resource, number = self.ensure_type(resource, number, thing=thing)
        thing.del_attrs(self._attr_name, number=number, subkey=key, value=value)


    def additional_attrs(self, thing, resource, number):
        pass


    def post_automatic_allocation(self, thing, resource, number):
        pass


    def post_allocation(self, thing, resource, number):
        pass

    
    def allocate(self, thing, resource=(), number=True, force=False):
        """allocates a resource element to the given thing.

        resource - is passed as an argument it will be checked 
                   before assignment.  

        refattr - the attribute name on the entity that will refer back
                  this resource manager.

        returns the resource that was either passed in and processed 
        or generated.
        """

        try:
            clusto.begin_transaction()
            if not isinstance(thing, Driver):
                raise TypeError("thing is not of type Driver")

            if resource is ():
                # allocate a new resource
                resource, number = self.allocator(thing)
                auto_allocated = True
            else:
                auto_allocated = False
                resource, number = self.ensure_type(resource, number, thing)
                if not force and not self.available(resource, number, thing):
                    raise ResourceException("Requested resource is not available.")

            if self._record_allocations:
                if number == True:
                    c = Counter.get(ClustoMeta().entity, self._attr_name)
                    attr = thing.add_attr(self._attr_name,
                                          resource,
                                          number=c.value
                                          )
                    c.next()
                else:
                    attr = thing.add_attr(self._attr_name, resource, number=number)
                    
                clusto.flush()

                a=thing.add_attr(self._attr_name,
                            self.entity,
                            number=attr.number,
                            subkey='manager',
                            )

                clusto.flush()
                self.additional_attrs(thing, resource, attr.number)
                
            else:
                attr = None
            clusto.commit()
        except Exception, x:
            clusto.rollback_transaction()
            raise

        if auto_allocated:
            self.post_automatic_allocation(thing, resource, number)

        self.post_allocation(thing, resource, number)
            
        return attr #resource


    def deallocate(self, thing, resource=(), number=True):
        """deallocates a resource from the given thing."""


        clusto.begin_transaction()
        try:
            if resource is ():                      
                for res in self.resources(thing):
                    thing.del_attrs(self._attr_name, number=res.number)

            elif resource and not self.available(resource, number):
                resource, number = self.ensure_type(resource, number)

                res = thing.attrs(self._attr_name, self, subkey='manager', number=number)
                for a in res: 
                    thing.del_attrs(self._attr_name, number=a.number)
                    
            clusto.commit()
        except Exception, x:
            clusto.rollback_transaction()
            raise

    def available(self, resource, number=True, thing=None):
        """return True if resource is available, False otherwise.
        """

        resource, number = self.ensure_type(resource, number)

        if self.owners(resource, number):
            return False

        return True
            

    def owners(self, resource, number=True):
        """return a list of driver objects for the owners of a given resource.
        """

        resource, number = self.ensure_type(resource, number)

        return Driver.get_by_attr(self._attr_name, resource, number=number)


    @classmethod
    def get_resource_manager(cls, resource_attr):
        """Return the resource manager for a given resource_attr"""

        thing = Driver(resource_attr.entity)
        
        return thing.attr_value(key=resource_attr.key,
                                subkey='manager',
                                number=resource_attr.number)
        

    @classmethod
    def resources(cls, thing):
        """return a list of resources from the resource manager that is
        associated with the given thing.

        A resource is a resource attribute in a resource manager.
        """
        
        attrs = [x for x in thing.attrs(cls._attr_name, subkey='manager') 
                 if isinstance(Driver(x.value), cls)]

        res = []

        for attr in attrs:
            t=thing.attrs(cls._attr_name, number=attr.number, subkey=None)
            res.extend(t)


        return res


    @property
    def count(self):
        """Return the number of resources used."""

        return len(self.references(self._attr_name, self, subkey='manager'))





import clusto
from clusto.drivers import BasicServer, IPManager
from clusto.test import testbase

from clusto.exceptions import ResourceException

class BasicServerTest(testbase.ClustoTestBase):

    def data(self):
        s1 = BasicServer('bs1', model='7000', manufacturer='ibm')
        s2 = BasicServer('bs2', model='ab1200', manufacturer='sun')

        
    def testBasicServerCreation(self):

        s1 = clusto.get_by_name('bs1')
        s2 = clusto.get_by_name('bs2')

        self.assertEqual(s1.model, '7000')
        self.assertEqual(s1.manufacturer, 'ibm')
        self.assertEqual(s2.model, 'ab1200')
        self.assertEqual(s2.manufacturer, 'sun')

        
    def testHostname(self):

        s1 = clusto.get_by_name('bs1')
        s2 = clusto.get_by_name('bs2')

        s2.hostname = "testname"

        clusto.flush()

        self.assertEqual(s1.hostname, "bs1")

        self.assertEqual(s2.hostname, "testname")

        self.assertEqual(s2.entity.name, "bs2")

        s2.hostname = "newname"

        self.assertEqual(s2.hostname, "newname")
        

    def testfqdn(self):

        s1 = clusto.get_by_name('bs1')
        s2 = clusto.get_by_name('bs2')

        self.assertEqual(s1.fqdns, [])

        s2.add_fqdn("test.example.com")

        self.assertEqual(["test.example.com"],
                         s2.fqdns)

        s2.add_fqdn("test2.example.com")
        
        clusto.flush()

        self.assertEqual(sorted(["test.example.com",
                                 "test2.example.com"]),
                         sorted(s2.fqdns))

        s2.remove_fqdn("test.example.com")

        
        self.assertEqual(["test2.example.com"],
                         s2.fqdns)


    def testBindingIPtoOSPort(self):

        s1 = clusto.get_by_name('bs1')
        s2 = clusto.get_by_name('bs2')
                
        ipm1 = IPManager('ipman1', netmask='255.255.255.0', baseip='192.168.1.0')
        ipm2 = IPManager('ipman2', netmask='255.255.255.0', baseip='192.168.2.0')

        s1.bind_ip_to_osport('192.168.1.20', 'eth0', porttype='nic-eth', portnum=1)
        s1.bind_ip_to_osport('192.168.2.20', 'eth1', porttype='nic-eth', portnum=2)

        
    def testAddingIP(self):

        s1 = clusto.get_by_name('bs1')

        self.assertRaises(ResourceException, s1.add_ip, '10.0.0.100')

        ipm = IPManager('ipman', netmask='255.255.0.0', baseip='10.0.0.1')

        s1.add_ip('10.0.0.100')
        
        self.assertTrue(s1.has_ip('10.0.0.100'))

        s1.add_ip(ipman=ipm)

        self.assertTrue(s1.has_ip('10.0.0.1'))

    def testAddingIPfromIPManagerWithGateway(self):
                        
        s1 = clusto.get_by_name('bs1')
        ipm = IPManager('ipman', netmask='255.255.0.0', baseip='10.0.0.1', gateway='10.0.0.1')

        s1.add_ip(ipman=ipm)

        self.assertTrue(s1.has_ip('10.0.0.2'))
        

    def testBindingIPtoOSPort(self):

        s1 = clusto.get_by_name('bs1')

        ipm1 = IPManager('ipman1', netmask='255.255.255.0', baseip='10.0.0.1', gateway='10.0.0.1')
        ipm2 = IPManager('ipman2', netmask='255.255.255.0', baseip='10.0.1.1', gateway='10.0.1.1')

        self.assertRaises(Exception, s1.bind_ip_to_osport, '10.0.0.100', 'eth0', porttype='nic-eth')
        self.assertRaises(Exception, s1.bind_ip_to_osport, '10.0.0.100', 'eth0', portnum=0)
        
        s1.bind_ip_to_osport('10.0.0.100', 'eth0')#, porttype='nic-eth', portnum=1)

        self.assertEqual(IPManager.get_devices('10.0.0.100'), [s1])
        
        self.assertRaises(Exception, s1.bind_ip_to_osport, '10.0.1.100', 'eth1', porttype='nic-eth')
        self.assertRaises(Exception, s1.bind_ip_to_osport, '10.0.1.100', 'eth1', portnum=1)
        
        s1.bind_ip_to_osport('10.0.1.100', 'eth1')#, porttype='nic-eth', portnum=2)

        self.assertEqual(IPManager.get_devices('10.0.1.100'), [s1])
        
    def testGettingByIP(self):
        s1 = clusto.get_by_name('bs1')
        ipm = IPManager('ipman', netmask='255.255.0.0', baseip='10.0.0.1')
        s1.add_ip(ipman=ipm)
        self.assertTrue(clusto.get_by_ip('10.0.0.1'), [s1])




from clusto.test import testbase

from clusto.drivers import BasicServer, BasicRack, IPManager
from clusto.drivers import BasicNetworkSwitch, BasicPowerStrip
from clusto.exceptions import ConnectionException
import clusto

import os
import threading
import ConfigParser

thread_count = 0

def barrier_creator(count):

    semaphore = threading.Semaphore()
    event = threading.Event()

    def synchronise():
        """ All calls to this method will block until the last (count) call is made """
        global thread_count

        semaphore.acquire()
        thread_count += 1
        if thread_count == count:
            event.set()
        semaphore.release()

        event.wait(3)

    return synchronise

    
class ClustoWorkThread(threading.Thread):

    def __init__(self, db, echo, barrier):
        super(ClustoWorkThread, self).__init__()  
        conf = ConfigParser.ConfigParser()
        conf.add_section('clusto')
        conf.set('clusto', 'dsn', db)
        self.conf = conf
        self.echo = echo
        self.barrier = barrier
        
    def run(self):

        clusto.connect(self.conf,echo=self.echo)
        clusto.init_clusto()

        try:

            clusto.begin_transaction()

            e = clusto.Entity('foo'+self.getName())

            self.barrier()

            clusto.commit()
        except Exception, x:
            clusto.rollback_transaction()
            raise x
            
class ConcurrentTest(testbase.unittest.TestCase):

    def setUp(self):
        conf = ConfigParser.ConfigParser()
        conf.add_section('clusto')
        conf.set('clusto', 'dsn', testbase.DB)

        clusto.SESSION.clusto_version = clusto.working_version()
        clusto.connect(conf,echo=testbase.ECHO)
        clusto.METADATA.drop_all(clusto.SESSION.bind)
        clusto.clear()
        clusto.SESSION.close()


    def tearDown(self):
        if clusto.SESSION.is_active:
            raise Exception("SESSION IS STILL ACTIVE in %s" % str(self.__class__))

        clusto.clear()
        clusto.disconnect()
        clusto.METADATA.drop_all(clusto.SESSION.bind)

        
    def testConcurrentThreads(self):

        DB = testbase.DB
        if DB.startswith('sqlite'):
            return
            
        conf = ConfigParser.ConfigParser()
        conf.add_section('clusto')
        conf.set('clusto', 'dsn', testbase.DB)
        clusto.connect(conf, echo=testbase.ECHO)
        clusto.init_clusto()
        firstver = clusto.get_latest_version_number()
        
        threadcount = 5
        threads = []
        barrier = barrier_creator(threadcount)
        for i in range(threadcount):
            threads.append(ClustoWorkThread(DB, testbase.ECHO,
                                            barrier))

        for i in threads:
            i.start()

        for i in threads:
            i.join()

        self.assertEqual(clusto.get_latest_version_number(),
                         threadcount+firstver)        


        



#!/usr/bin/env python
# -*- mode: python; sh-basic-offset: 4; indent-tabs-mode: nil; coding: utf-8 -*-
# vim: tabstop=4 softtabstop=4 expandtab shiftwidth=4 fileencoding=utf-8

import argparse
import sys

import clusto
from clusto import drivers
from clusto import script_helper

class Reboot(script_helper.Script):
    '''
    This will reboot a given server or IP address
    '''

    def __init__(self):
        script_helper.Script.__init__(self)

    def _add_arguments(self, parser):
        parser.add_argument('object', nargs=1,
            help='Server name or IP address')

    def add_subparser(self, subparsers):
        parser = self._setup_subparser(subparsers)
        self._add_arguments(parser)

    def run(self, args):
        try:
            obj = clusto.get(args.object[0])
        except Exception as e:
            self.debug(e)
            self.error('"%s" does not exist' % args.object[0])
            return -1
        obj = obj[0]
        if not isinstance(obj, drivers.BasicServer):
            self.error('Seems like "%s" is not a server (type=%s), you cannot FAI this' % (args.object[0], obj.type))
            return 1
        self.debug('Working with %s' % obj)
        print 'Parents: %s' % ' '.join([ _.name for _ in obj.parents() ])
        sys.stdout.write('Are you sure you want to reboot %s (yes/No)? ' % args.object[0])
        sys.stdout.flush()

        try:
            line = sys.stdin.readline().rstrip('\r\n')
        except KeyboardInterrupt:
            line = False

        if line != 'yes':
            print 'Aborting'
            return
        else:
            obj.power_reboot(captcha=False)
            print '%s is rebooting now.' % obj.name
        return


def main():
    reboot = Reboot()
    parent_parser = script_helper.setup_base_parser()
    this_parser = argparse.ArgumentParser(parents=[parent_parser],
        description=reboot._get_description())
    reboot._add_arguments(this_parser)
    args = this_parser.parse_args()
    reboot.init_script(args=args, logger=script_helper.get_logger(args.loglevel))
    return(reboot.run(args))


if __name__ == '__main__':
    sys.exit(main())




#!/usr/bin/env python
# -*- mode: python; sh-basic-offset: 4; indent-tabs-mode: nil; coding: utf-8 -*-
# vim: tabstop=4 softtabstop=4 expandtab shiftwidth=4 fileencoding=utf-8

import argparse
import sys

import clusto
from clusto import drivers
from clusto import script_helper
from pprint import pprint
import sys
import traceback


JSON=False
YAML=False
try:
    import yaml
    YAML=True
except ImportError:
    pass

try:
    import simplejson as json
    JSON=True
except ImportError:
    try:
        import json
        JSON=True
    except:
        pass


class Attr(script_helper.Script):
    '''
    Operate upon an object's attributes, you should be able to
    add, remove, list or set attributes of any kind
    '''

    obj = None
    format = 'list'

    def __init__(self):
        script_helper.Script.__init__(self)

    def run_show_yaml(self, attrs):
        self.debug('Printing in format: YAML')
        print yaml.safe_dump(attrs, encoding='utf-8',
            explicit_start=True, default_flow_style=False)
        return 0

    def run_show_json(self, attrs):
        self.debug('Printing in format: JSON')
        print json.dumps(attrs, sort_keys=True, indent=2)
        return 0

    def run_show_csv(self, attrs):
        self.debug('Printing in format: CSV')
        print 'key;subkey;number;"value"'
        for attr in attrs:
            print '%s;%s;%s;"%s"' % (
                str(attr['key'] or ''),
                str(attr['subkey'] or ''),
                str(int(attr['number'] or 0)),
                str(attr['value']))
        return 0

    def run_show_list(self, attrs):
        self.debug('Printing in format: List')
        maxkey = 3 + max([len(str(_['key'])) for _ in attrs] + [0])
        maxsubkey = 6 + max([len(str(_['subkey'])) for _ in attrs] + [0])
        maxnumber = 3 + max([len(str(_['number'])) for _ in attrs] + [0])

        if maxkey < 5: maxkey = 5
        if maxsubkey < 8: maxsubkey = 8

        print ''.join(['KEY'.ljust(maxkey, ' '), 'SUBKEY'.ljust(maxsubkey, ' '), 'VALUE'])
        for attr in attrs:
            print ''.join([str(_).ljust(maxsize, ' ') for _, maxsize in [
                (attr['key'], maxkey),
                (attr['subkey'], maxsubkey),
                (attr['value'], 0),
            ]])
        return 0

    def run_set(self, kwargs):
        kwargs.pop('merge_container_attrs')
        return self.obj.set_attr(**kwargs)

    def run_add(self, kwargs):
        kwargs.pop('merge_container_attrs')
        return self.obj.add_attr(**kwargs)

    def run_delete(self, kwargs):
        kwargs.pop('merge_container_attrs')
        return self.obj.del_attrs(**kwargs)

    def run_show(self, kwargs):
        attrs = self.obj.attrs(**kwargs)
        attrs.sort(key=lambda _: (_.key, _.number, _.subkey, _.value))
        result = []
        for attr in attrs:
            row = {
                'key': attr.key,
                'subkey': attr.subkey,
                'number': attr.number,
                'type': attr.datatype,
                'value': unicode(attr.value)
            }
            result.append(row)
        return (getattr(self, 'run_show_%s' % self.format)(result))

    def run(self, args):
        obj = clusto.get(args.obj[0])
        if not obj:
            self.error('Object %s does not exist' % args.obj[0])
            return -1
        self.obj = obj[0]
        opts = {}
        kwargs = dict(args.__dict__.items())
        self.format = args.format
        for k in ['key', 'subkey', 'value', 'merge_container_attrs']:
            if kwargs[k] != None:
                opts[k] = kwargs[k]
        return (getattr(self, 'run_%s' % args.action[0])(opts))

    def _add_arguments(self, parser):
        actions = ['add', 'show', 'set', 'delete']
        choices = ['list', 'csv']
        if JSON:
            choices.append('json')
        if YAML:
            choices.append('yaml')
        parser.add_argument('action', nargs=1, metavar='action', choices=actions,
            help='Action to execute (add, delete, set, show)')
        parser.add_argument('--format', choices=choices, default='list',
            help='What format to use to display the info, defaults to "list"')
        parser.add_argument('-k', '--key', help='Attribute key to filter on',
            default=None)
        parser.add_argument('-s', '--subkey', help='Attribute subkey to filter on',
            default=None)
        parser.add_argument('-v', '--value', help='Attribute value to filter on',
            default=None)
        parser.add_argument('-m', '--merge', default=False, action='store_true',
            dest='merge_container_attrs',
            help='Merge container attributes recursively (defaults to False)')
        parser.add_argument('obj', nargs=1, metavar='object',
            help='Object to modify/query attributes from')


def main():
    attr, args = script_helper.init_arguments(Attr)
    return(attr.run(args))

if __name__ == '__main__':
    sys.exit(main())

